# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from ObjCDataFormatterTestCase import ObjCDataFormatterTestCase


class ObjCDataFormatterNSException(ObjCDataFormatterTestCase):
    SHARED_BUILD_TESTCASE = False

    def test_nsexception_with_run_command(self):
        """Test formatters for NSException."""
        self.appkit_tester_impl(self.nsexception_data_formatter_commands, True)

    @skipUnlessDarwin
    def test_nsexception_with_run_command_no_const(self):
        """Test formatters for NSException."""
        self.appkit_tester_impl(self.nsexception_data_formatter_commands, False)

    def nsexception_data_formatter_commands(self):
        self.expect(
            "frame variable except0 except1 except2 except3",
            substrs=[
                "(NSException *) except0 = ",
                '@"First"',
                "(NSException *) except1 = ",
                '@"Second"',
                "(NSException *) except2 = ",
                ' @"Third"',
                "(NSException *) except3 = ",
                ' @"Fourth"',
            ],
        )



import lldb
import binascii
import os
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


@skipIfRemote
class TestProcessConnect(GDBRemoteTestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_gdb_remote_sync(self):
        """Test the gdb-remote command in synchronous mode"""
        try:
            self.dbg.SetAsync(False)
            self.expect(
                "gdb-remote " + self.server.get_connect_address(),
                substrs=["Process", "stopped"],
            )
        finally:
            self.dbg.GetSelectedTarget().GetProcess().Kill()

    def test_gdb_remote_async(self):
        """Test the gdb-remote command in asynchronous mode"""
        try:
            self.dbg.SetAsync(True)
            self.expect(
                "gdb-remote " + self.server.get_connect_address(),
                matching=False,
                substrs=["Process", "stopped"],
            )
            lldbutil.expect_state_changes(
                self, self.dbg.GetListener(), self.process(), [lldb.eStateStopped]
            )
        finally:
            self.dbg.GetSelectedTarget().GetProcess().Kill()
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), self.process(), [lldb.eStateExited]
        )

    @skipIfWindows
    def test_process_connect_sync(self):
        """Test the gdb-remote command in synchronous mode"""
        try:
            self.dbg.SetAsync(False)
            self.expect(
                "platform select remote-gdb-server",
                substrs=["Platform: remote-gdb-server", "Connected: no"],
            )
            self.expect(
                "process connect " + self.server.get_connect_url(),
                substrs=["Process", "stopped"],
            )
        finally:
            self.dbg.GetSelectedTarget().GetProcess().Kill()

    @skipIfWindows
    def test_process_connect_async(self):
        """Test the gdb-remote command in asynchronous mode"""
        try:
            self.dbg.SetAsync(True)
            self.expect(
                "platform select remote-gdb-server",
                substrs=["Platform: remote-gdb-server", "Connected: no"],
            )
            self.expect(
                "process connect " + self.server.get_connect_url(),
                matching=False,
                substrs=["Process", "stopped"],
            )
            lldbutil.expect_state_changes(
                self, self.dbg.GetListener(), self.process(), [lldb.eStateStopped]
            )
        finally:
            self.dbg.GetSelectedTarget().GetProcess().Kill()
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), self.process(), [lldb.eStateExited]
        )

    def test_breakpoint_count(self):
        """
        Test that breakpoint count gets reset for each new connection.
        """

        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                super().__init__()
                self.continued = False

            def qfThreadInfo(self):
                return "m47"

            def qsThreadInfo(self):
                return "l"

            def setBreakpoint(self, packet):
                return "OK"

            def readRegister(self, reg):
                # Pretend we're at the breakpoint after we've been resumed.
                return "3412000000000000" if self.continued else "4747000000000000"

            def cont(self):
                self.continued = True
                return "T05thread=47;reason:breakpoint"

        # Connect to the first process and set our breakpoint.
        self.server.responder = MyResponder()
        target = self.createTarget("a.yaml")
        process = self.connect(target)

        bkpt = target.BreakpointCreateByAddress(0x1234)
        self.assertTrue(bkpt.IsValid())
        self.assertEqual(bkpt.GetNumLocations(), 1)

        # "continue" the process. It should hit our breakpoint.
        process.Continue()
        self.assertState(process.GetState(), lldb.eStateStopped)
        self.assertEqual(bkpt.GetHitCount(), 1)

        # Now kill it. The breakpoint should still show a hit count of one.
        process.Kill()
        self.server.stop()
        self.assertEqual(bkpt.GetHitCount(), 1)

        # Start over, and reconnect.
        self.server = MockGDBServer(self.server_socket_class())
        self.server.start()

        process = self.connect(target)

        # The hit count should be reset.
        self.assertEqual(bkpt.GetHitCount(), 0)



"""
Test watchpoint slots we should not be able to install multiple watchpoints
within same word boundary. We should be able to install individual watchpoints
on any of the bytes, half-word, or word. This is only for ARM/AArch64 targets.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchpointSlotsTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Source filename.
        self.source = "main.c"

        # Output filename.
        self.exe_name = self.getBuildArtifact("a.out")
        self.d = {"C_SOURCES": self.source, "EXE": self.exe_name}

    # This is a arm and aarch64 specific test case. No other architectures tested.
    @skipIf(archs=no_match(["arm$", "aarch64"]))
    def test_multiple_watchpoints_on_same_word(self):
        self.build(dictionary=self.d)
        self.setTearDownCleanup(dictionary=self.d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Detect line number after which we are going to increment arrayName.
        loc_line = line_number("main.c", "// About to write byteArray")

        # Set a breakpoint on the line detected above.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", loc_line, num_expected_locations=1, loc_exact=True
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Delete breakpoint we just hit.
        self.expect("breakpoint delete 1", substrs=["1 breakpoints deleted"])

        # Set a watchpoint at byteArray[0]
        self.expect(
            "watchpoint set variable byteArray[0]",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 1"],
        )

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v 1", substrs=["hit_count = 0"])

        # debugserver on ios doesn't give an error, it creates another watchpoint,
        # only expect errors on non-darwin platforms.
        if not self.platformIsDarwin():
            # Try setting a watchpoint at byteArray[1]
            self.expect(
                "watchpoint set variable byteArray[1]",
                error=True,
                substrs=["Watchpoint creation failed"],
            )

        self.runCmd("process continue")

        # We should be stopped due to the watchpoint.
        # The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "stop reason = watchpoint 1"],
        )

        # Delete the watchpoint we hit above successfully.
        self.expect("watchpoint delete 1", substrs=["1 watchpoints deleted"])

        # Set a watchpoint at byteArray[3]
        self.expect(
            "watchpoint set variable byteArray[3]",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 1"],
        )

        # Resume inferior.
        self.runCmd("process continue")

        # We should be stopped due to the watchpoint.
        # The stop reason of the thread should be watchpoint.
        if self.platformIsDarwin():
            # On darwin we'll hit byteArray[3] which is watchpoint 2
            self.expect(
                "thread list",
                STOPPED_DUE_TO_WATCHPOINT,
                substrs=["stopped", "stop reason = watchpoint 2"],
            )
        else:
            self.expect(
                "thread list",
                STOPPED_DUE_TO_WATCHPOINT,
                substrs=["stopped", "stop reason = watchpoint 3"],
            )

        # Resume inferior.
        self.runCmd("process continue")



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import expectedFailureWindows
import lldbdap_testcase


class TestDAP_launch(lldbdap_testcase.DAPTestCaseBase):
    def test_environment_with_object(self):
        """
        Tests launch of a simple program with environment variables
        """
        program = self.getBuildArtifact("a.out")
        env = {
            "NO_VALUE": "",
            "WITH_VALUE": "BAR",
            "EMPTY_VALUE": "",
            "SPACE": "Hello World",
        }

        self.build_and_launch(program, env=env)
        self.continue_to_exit()

        # Now get the STDOUT and verify our arguments got passed correctly
        output = self.get_stdout()
        self.assertTrue(output and len(output) > 0, "expect program output")
        lines = output.splitlines()
        # Skip the all arguments so we have only environment vars left
        while len(lines) and lines[0].startswith("arg["):
            lines.pop(0)
        # Make sure each environment variable in "env" is actually set in the
        # program environment that was printed to STDOUT
        for var in env:
            found = False
            for program_var in lines:
                if var in program_var:
                    found = True
                    break
            self.assertTrue(
                found, '"%s" must exist in program environment (%s)' % (var, lines)
            )



"""
Test the AArch64 SME ZA and ZT0 registers are saved and restored around
expressions.

This attempts to cover expressions that change the following:
* ZA enabled or not.
* Streaming mode or not.
* Streaming vector length (increasing and decreasing).
* Some combintations of the above.
"""

from enum import IntEnum
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


# These enum values match the flag values used in the test program.
class Mode(IntEnum):
    SVE = 0
    SSVE = 1


class ZA(IntEnum):
    Disabled = 0
    Enabled = 1


class AArch64ZATestCase(TestBase):
    def get_supported_svg(self):
        # Always build this probe program to start as streaming SVE.
        # We will read/write "vg" here but since we are in streaming mode "svg"
        # is really what we are writing ("svg" is a read only pseudo).
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        # Enter streaming mode, don't enable ZA, start_vl and other_vl don't
        # matter here.
        self.runCmd("settings set target.run-args 1 0 0 0")

        stop_line = line_number("main.c", "// Set a break point here.")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", stop_line, num_expected_locations=1
        )

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        # Write back the current vg to confirm read/write works at all.
        current_svg = self.match("register read vg", ["(0x[0-9]+)"])
        self.assertIsNotNone(current_svg)
        self.expect("register write vg {}".format(current_svg.group()))

        # Aka 128, 256 and 512 bit.
        supported_svg = []
        for svg in [2, 4, 8]:
            # This could mask other errors but writing vg is tested elsewhere
            # so we assume the hardware rejected the value.
            self.runCmd("register write vg {}".format(svg), check=False)
            if not self.res.GetError():
                supported_svg.append(svg)

        self.runCmd("breakpoint delete 1")
        self.runCmd("continue")

        return supported_svg

    def read_vg(self):
        process = self.dbg.GetSelectedTarget().GetProcess()
        registerSets = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        sve_registers = registerSets.GetFirstValueByName(
            "Scalable Vector Extension Registers"
        )
        return sve_registers.GetChildMemberWithName("vg").GetValueAsUnsigned()

    def read_svg(self):
        process = self.dbg.GetSelectedTarget().GetProcess()
        registerSets = process.GetThreadAtIndex(0).GetFrameAtIndex(0).GetRegisters()
        sve_registers = registerSets.GetFirstValueByName(
            "Scalable Matrix Extension Registers"
        )
        return sve_registers.GetChildMemberWithName("svg").GetValueAsUnsigned()

    def make_za_value(self, vl, generator):
        # Generate a vector value string "{0x00 0x01....}".
        rows = []
        for row in range(vl):
            byte = "0x{:02x}".format(generator(row))
            rows.append(" ".join([byte] * vl))
        return "{" + " ".join(rows) + "}"

    def check_za(self, vl):
        # We expect an increasing value starting at 1. Row 0=1, row 1 = 2, etc.
        self.expect(
            "register read za", substrs=[self.make_za_value(vl, lambda row: row + 1)]
        )

    def check_za_disabled(self, vl):
        # When ZA is disabled, lldb will show ZA as all 0s.
        self.expect("register read za", substrs=[self.make_za_value(vl, lambda row: 0)])

    def make_zt0_value(self, generator):
        num_bytes = 512 // 8
        elements = []
        for i in range(num_bytes):
            elements.append("0x{:02x}".format(generator(i)))

        return "{" + " ".join(elements) + "}"

    def check_zt0(self):
        self.expect("register read zt0", substrs=[self.make_zt0_value(lambda n: n + 1)])

    def check_zt0_disabled(self):
        # Like ZA, zt0 reads as 0 when SVCR.ZA is not set.
        self.expect("register read zt0", substrs=[self.make_zt0_value(lambda n: 0)])

    def za_expr_test_impl(self, sve_mode, za_state, swap_start_vl):
        if not self.isAArch64SMEFA64():
            self.skipTest("SME and the smefa64 extension must be present.")

        supported_svg = self.get_supported_svg()
        if len(supported_svg) < 2:
            self.skipTest("Target must support at least 2 streaming vector lengths.")

        # vg is in units of 8 bytes.
        start_vl = supported_svg[1] * 8
        other_vl = supported_svg[2] * 8

        if swap_start_vl:
            start_vl, other_vl = other_vl, start_vl

        self.line = line_number("main.c", "// Set a break point here.")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        self.runCmd(
            "settings set target.run-args {} {} {} {}".format(
                sve_mode, za_state, start_vl, other_vl
            )
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        exprs = [
            "expr_disable_za",
            "expr_enable_za",
            "expr_start_vl",
            "expr_other_vl",
            "expr_enable_sm",
            "expr_disable_sm",
        ]

        # This may be the streaming or non-streaming vg. All that matters is
        # that it is saved and restored, remaining constant throughout.
        start_vg = self.read_vg()

        # Check SVE registers to make sure that combination of scaling SVE
        # and scaling ZA works properly. This is a brittle check, but failures
        # are likely to be catastrophic when they do happen anyway.
        sve_reg_names = "ffr {} {}".format(
            " ".join(["z{}".format(n) for n in range(32)]),
            " ".join(["p{}".format(n) for n in range(16)]),
        )
        self.runCmd("register read " + sve_reg_names)
        sve_values = self.res.GetOutput()

        za = 1 if za_state == ZA.Enabled else 0
        sm = 1 if sve_mode == Mode.SSVE else 0
        svcr_value = "0x{:016x}".format((za << 1) | sm)
        expected_svcr = [svcr_value]
        if self.hasXMLSupport():
            expected_svcr.append("(ZA = {}, SM = {})".format(za, sm))

        has_zt0 = self.isAArch64SME2()

        def check_regs():
            if za_state == ZA.Enabled:
                self.check_za(start_vl)
                if has_zt0:
                    self.check_zt0()
            else:
                self.check_za_disabled(start_vl)
                if has_zt0:
                    self.check_zt0_disabled()

            # svg and vg are in units of 8 bytes.
            self.assertEqual(start_vl, self.read_svg() * 8)
            self.assertEqual(start_vg, self.read_vg())

            self.expect("register read " + sve_reg_names, substrs=[sve_values])

            self.expect("register read svcr", substrs=expected_svcr)

        for expr in exprs:
            expr_cmd = "expression {}()".format(expr)

            # We do this twice because there were issues in development where
            # using data stored by a previous WriteAllRegisterValues would crash
            # the second time around.
            self.runCmd(expr_cmd)
            check_regs()
            self.runCmd(expr_cmd)
            check_regs()

        # Run them in sequence to make sure there is no state lingering between
        # them after a restore.
        for expr in exprs:
            self.runCmd("expression {}()".format(expr))
            check_regs()

        for expr in reversed(exprs):
            self.runCmd("expression {}()".format(expr))
            check_regs()

    # These tests start with the 1st supported SVL and change to the 2nd
    # supported SVL as needed.

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_ssve_za_enabled(self):
        self.za_expr_test_impl(Mode.SSVE, ZA.Enabled, False)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_ssve_za_disabled(self):
        self.za_expr_test_impl(Mode.SSVE, ZA.Disabled, False)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_sve_za_enabled(self):
        self.za_expr_test_impl(Mode.SVE, ZA.Enabled, False)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_sve_za_disabled(self):
        self.za_expr_test_impl(Mode.SVE, ZA.Disabled, False)

    # These tests start in the 2nd supported SVL and change to the 1st supported
    # SVL as needed.

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_ssve_za_enabled_different_vl(self):
        self.za_expr_test_impl(Mode.SSVE, ZA.Enabled, True)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_ssve_za_disabled_different_vl(self):
        self.za_expr_test_impl(Mode.SSVE, ZA.Disabled, True)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_sve_za_enabled_different_vl(self):
        self.za_expr_test_impl(Mode.SVE, ZA.Enabled, True)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_za_expr_sve_za_disabled_different_vl(self):
        self.za_expr_test_impl(Mode.SVE, ZA.Disabled, True)



# encoding: utf-8
"""
Test lldb's frame recognizers.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import recognizer


class FrameRecognizerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_frame_recognizer_1(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "foo", exe_name=exe
        )
        frame = thread.selected_frame

        # Clear internal & plugins recognizers that get initialized at launch
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        self.expect("frame recognizer list", substrs=["no matching results found."])

        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo"
        )

        self.expect(
            "frame recognizer list",
            substrs=[
                "0: recognizer.MyFrameRecognizer, module a.out, demangled symbol foo"
            ],
        )

        self.runCmd(
            "frame recognizer add -l recognizer.MyOtherFrameRecognizer -s a.out -n bar -x"
        )

        self.expect(
            "frame recognizer list",
            substrs=[
                "1: recognizer.MyOtherFrameRecognizer, module a.out, demangled symbol regex bar",
                "0: recognizer.MyFrameRecognizer, module a.out, demangled symbol foo",
            ],
        )

        self.runCmd("frame recognizer delete 0")

        # Test that it deleted the recognizer with id 0.
        self.expect(
            "frame recognizer list",
            substrs=[
                "1: recognizer.MyOtherFrameRecognizer, module a.out, demangled symbol regex bar"
            ],
        )
        self.expect(
            "frame recognizer list", matching=False, substrs=["MyFrameRecognizer"]
        )

        # Test that an invalid index and deleting the same index again
        # is an error and doesn't do any changes.
        self.expect(
            "frame recognizer delete 2",
            error=True,
            substrs=["error: '2' is not a valid recognizer id."],
        )
        self.expect(
            "frame recognizer delete 0",
            error=True,
            substrs=["error: '0' is not a valid recognizer id."],
        )
        # Recognizers should have the same state as above.
        self.expect(
            "frame recognizer list",
            substrs=[
                "1: recognizer.MyOtherFrameRecognizer, module a.out, demangled symbol regex bar"
            ],
        )
        self.expect(
            "frame recognizer list", matching=False, substrs=["MyFrameRecognizer"]
        )

        self.runCmd("frame recognizer clear")

        self.expect("frame recognizer list", substrs=["no matching results found."])

        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo"
        )

        self.expect("frame variable", substrs=["(int) a = 42", "(int) b = 56"])

        # Recognized arguments don't show up by default...
        variables = frame.GetVariables(lldb.SBVariablesOptions())
        self.assertEqual(variables.GetSize(), 0)

        # ...unless you set target.display-recognized-arguments to 1...
        self.runCmd("settings set target.display-recognized-arguments 1")
        variables = frame.GetVariables(lldb.SBVariablesOptions())
        self.assertEqual(variables.GetSize(), 2)

        # ...and you can reset it back to 0 to hide them again...
        self.runCmd("settings set target.display-recognized-arguments 0")
        variables = frame.GetVariables(lldb.SBVariablesOptions())
        self.assertEqual(variables.GetSize(), 0)

        # ... or explicitly ask for them with SetIncludeRecognizedArguments(True).
        opts = lldb.SBVariablesOptions()
        opts.SetIncludeRecognizedArguments(True)
        variables = frame.GetVariables(opts)

        self.assertEqual(variables.GetSize(), 2)
        self.assertEqual(variables.GetValueAtIndex(0).name, "a")
        self.assertEqual(variables.GetValueAtIndex(0).signed, 42)
        self.assertEqual(
            variables.GetValueAtIndex(0).GetValueType(), lldb.eValueTypeVariableArgument
        )
        self.assertEqual(variables.GetValueAtIndex(1).name, "b")
        self.assertEqual(variables.GetValueAtIndex(1).signed, 56)
        self.assertEqual(
            variables.GetValueAtIndex(1).GetValueType(), lldb.eValueTypeVariableArgument
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

        self.expect(
            "frame recognizer info 999", error=True, substrs=["no frame with index 999"]
        )

        self.expect(
            "frame recognizer info 1",
            substrs=["frame 1 not recognized by any recognizer"],
        )

        # FIXME: The following doesn't work yet, but should be fixed.
        """
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(self, "bar",
                                                                 exe_name = exe)
        frame = thread.GetSelectedFrame()

        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped', 'stop reason = breakpoint'])

        self.expect("frame variable -t",
                    substrs=['(int *) a = '])

        self.expect("frame variable -t *a",
                    substrs=['*a = 78'])
        """

    def test_frame_recognizer_hiding(self):
        self.build()

        target, process, thread, _ = lldbutil.run_to_name_breakpoint(self, "nested")
        frame = thread.selected_frame

        # Sanity check.
        self.expect(
            "thread backtrace", patterns=["frame.*nested", "frame.*baz", "frame.*main"]
        )

        self.expect("frame recognizer clear")
        self.expect(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        self.expect(
            "frame recognizer add -l recognizer.BazFrameRecognizer -f false -s a.out -n baz"
        )

        self.expect(
            "frame recognizer list",
            substrs=["0: recognizer.BazFrameRecognizer"],
        )

        # Now main should be hidden.
        self.expect("thread backtrace", matching=False, patterns=["frame.*baz"])
        self.assertFalse(frame.IsHidden())
        frame = thread.SetSelectedFrame(1)
        self.assertIn("baz", frame.name)
        self.assertTrue(frame.IsHidden())

        # Test StepOut.
        frame = thread.SetSelectedFrame(0)
        thread.StepOut()
        frame = thread.GetSelectedFrame()
        self.assertIn("main", frame.name)

    def test_frame_recognizer_multi_symbol(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Clear internal & plugins recognizers that get initialized at launch
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        self.expect("frame recognizer list", substrs=["no matching results found."])

        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar"
        )

        self.expect(
            "frame recognizer list",
            substrs=[
                "recognizer.MyFrameRecognizer, module a.out, demangled symbol foo, bar"
            ],
        )

        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "foo", exe_name=exe
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "bar", exe_name=exe
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

    def test_frame_recognizer_target_specific(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Clear internal & plugins recognizers that get initialized at launch
        self.runCmd("frame recognizer clear")

        # Create a target.
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "foo", exe_name=exe
        )

        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        # Check that this doesn't contain our own FrameRecognizer somehow.
        self.expect(
            "frame recognizer list", matching=False, substrs=["MyFrameRecognizer"]
        )

        # Add a frame recognizer in that target.
        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar"
        )

        self.expect(
            "frame recognizer list",
            substrs=[
                "recognizer.MyFrameRecognizer, module a.out, demangled symbol foo, bar"
            ],
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

        # Create a second target. That one shouldn't have the frame recognizer.
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "bar", exe_name=exe
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 not recognized by any recognizer"],
        )

        # Add a frame recognizer to the new target.
        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n bar"
        )

        self.expect(
            "frame recognizer list",
            substrs=[
                "recognizer.MyFrameRecognizer, module a.out, demangled symbol bar"
            ],
        )

        # Now the new target should also recognize the frame.
        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

    def test_frame_recognizer_not_only_first_instruction(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Clear internal & plugins recognizers that get initialized at launch.
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        self.expect("frame recognizer list", substrs=["no matching results found."])

        # Create a target.
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "foo", exe_name=exe
        )

        # Move the PC one instruction further.
        self.runCmd("next")

        # Add a frame recognizer in that target.
        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar"
        )

        # It's not applied to foo(), because frame's PC is not at the first instruction of the function.
        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 not recognized by any recognizer"],
        )

        # Add a frame recognizer with --first-instruction-only=true.
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar --first-instruction-only=true"
        )

        # It's not applied to foo(), because frame's PC is not at the first instruction of the function.
        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 not recognized by any recognizer"],
        )

        # Now add a frame recognizer with --first-instruction-only=false.
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar --first-instruction-only=false"
        )

        # This time it should recognize the frame.
        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

        opts = lldb.SBVariablesOptions()
        opts.SetIncludeRecognizedArguments(True)
        frame = thread.selected_frame
        variables = frame.GetVariables(opts)

        self.assertEqual(variables.GetSize(), 2)
        self.assertEqual(variables.GetValueAtIndex(0).name, "a")
        self.assertEqual(variables.GetValueAtIndex(0).signed, 42)
        self.assertEqual(
            variables.GetValueAtIndex(0).GetValueType(), lldb.eValueTypeVariableArgument
        )
        self.assertEqual(variables.GetValueAtIndex(1).name, "b")
        self.assertEqual(variables.GetValueAtIndex(1).signed, 56)
        self.assertEqual(
            variables.GetValueAtIndex(1).GetValueType(), lldb.eValueTypeVariableArgument
        )

    def test_frame_recognizer_disable(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(
            self, "foo", exe_name=exe
        )

        # Clear internal & plugins recognizers that get initialized at launch.
        self.runCmd("frame recognizer clear")

        self.runCmd(
            "command script import "
            + os.path.join(self.getSourceDir(), "recognizer.py")
        )

        # Add a frame recognizer in that target.
        self.runCmd(
            "frame recognizer add -l recognizer.MyFrameRecognizer -s a.out -n foo -n bar"
        )

        # The frame is recognized
        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

        # Disable the recognizer
        self.runCmd("frame recognizer disable 0")

        self.expect(
            "frame recognizer list",
            substrs=[
                "0: [disabled] recognizer.MyFrameRecognizer, module a.out, demangled symbol foo"
            ],
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 not recognized by any recognizer"],
        )

        # Re-enable the recognizer
        self.runCmd("frame recognizer enable 0")

        self.expect(
            "frame recognizer list",
            substrs=[
                "0: recognizer.MyFrameRecognizer, module a.out, demangled symbol foo"
            ],
        )

        self.expect(
            "frame recognizer info 0",
            substrs=["frame 0 is recognized by recognizer.MyFrameRecognizer"],
        )

    @no_debug_info_test
    def test_frame_recognizer_delete_invalid_arg(self):
        self.expect(
            "frame recognizer delete a",
            error=True,
            substrs=["error: 'a' is not a valid recognizer id."],
        )
        self.expect(
            'frame recognizer delete ""',
            error=True,
            substrs=["error: '' is not a valid recognizer id."],
        )
        self.expect(
            "frame recognizer delete -1",
            error=True,
            substrs=["error: '-1' is not a valid recognizer id."],
        )
        self.expect(
            "frame recognizer delete 4294967297",
            error=True,
            substrs=["error: '4294967297' is not a valid recognizer id."],
        )

    @no_debug_info_test
    def test_frame_recognizer_info_invalid_arg(self):
        self.expect(
            "frame recognizer info a",
            error=True,
            substrs=["error: 'a' is not a valid frame index."],
        )
        self.expect(
            'frame recognizer info ""',
            error=True,
            substrs=["error: '' is not a valid frame index."],
        )
        self.expect(
            "frame recognizer info -1",
            error=True,
            substrs=["error: '-1' is not a valid frame index."],
        )
        self.expect(
            "frame recognizer info 4294967297",
            error=True,
            substrs=["error: '4294967297' is not a valid frame index."],
        )

    @no_debug_info_test
    def test_frame_recognizer_add_invalid_arg(self):
        self.expect(
            "frame recognizer add -f",
            error=True,
            substrs=["error: last option requires an argument"],
        )
        self.expect(
            "frame recognizer add -f -1",
            error=True,
            substrs=["error: invalid boolean value '-1' passed for -f option"],
        )
        self.expect(
            "frame recognizer add -f foo",
            error=True,
            substrs=["error: invalid boolean value 'foo' passed for -f option"],
        )



"""
Test that we do not attempt to make a dynamic type for a 'const char*'
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCaseCharStarDynType(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

    def test_charstar_dyntype(self):
        """Test that we do not attempt to make a dynamic type for a 'const char*'"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here.", lldb.SBFileSpec("main.m")
        )

        # check that we correctly see the const char*, even with dynamic types
        # on
        self.expect("frame variable -raw-output my_string", substrs=["const char *"])
        self.expect(
            "frame variable my_string --raw-output --dynamic-type run-target",
            substrs=["const char *"],
        )
        # check that expr also gets it right
        self.expect("expr -R -- my_string", substrs=["const char *"])
        self.expect("expr -R -d run -- my_string", substrs=["const char *"])
        # but check that we get the real Foolie as such
        self.expect("frame variable my_foolie", substrs=["FoolMeOnce *"])
        self.expect(
            "frame variable my_foolie --dynamic-type run-target",
            substrs=["FoolMeOnce *"],
        )
        # check that expr also gets it right
        self.expect("expr my_foolie", substrs=["FoolMeOnce *"])
        self.expect("expr -d run -- my_foolie", substrs=["FoolMeOnce *"])
        # now check that assigning a true string does not break anything
        self.runCmd("next")
        # check that we correctly see the const char*, even with dynamic types
        # on
        self.expect("frame variable my_string", substrs=["const char *"])
        self.expect(
            "frame variable my_string --dynamic-type run-target",
            substrs=["const char *"],
        )
        # check that expr also gets it right
        self.expect("expr my_string", substrs=["const char *"])
        self.expect("expr -d run -- my_string", substrs=["const char *"])
        # but check that we get the real Foolie as such
        self.expect("frame variable my_foolie", substrs=["FoolMeOnce *"])
        self.expect(
            "frame variable my_foolie --dynamic-type run-target",
            substrs=["FoolMeOnce *"],
        )
        # check that expr also gets it right
        self.expect("expr my_foolie", substrs=["FoolMeOnce *"])
        self.expect("expr -d run -- my_foolie", substrs=["FoolMeOnce *"])



"""
Test that debug symbols have the correct order as specified by the order file.
"""


import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class OrderFileTestCase(TestBase):
    @skipUnlessDarwin
    def test(self):
        """Test debug symbols follow the correct order by the order file."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Test that the debug symbols have Function f3 before Function f1.
        # Use "-s address" option to sort by address.
        self.runCmd("image dump symtab -s address %s" % exe)
        output = self.res.GetOutput()
        mo_f3 = re.search("Code +.+f3", output)
        mo_f1 = re.search("Code +.+f1", output)

        # Match objects for f3 and f1 must exist and f3 must come before f1.
        self.assertTrue(
            mo_f3 and mo_f1 and mo_f3.start() < mo_f1.start(),
            "Symbols have correct order by the order file",
        )

        self.runCmd("run", RUN_COMPLETED)



from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentCrashWithSignal(ConcurrentEventsBase):
    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple="^mips")
    def test(self):
        """Test a thread that crashes while another thread generates a signal."""
        self.build()
        self.do_thread_actions(num_crash_threads=1, num_signal_threads=1)



"""
Read in a library with a version number of 0.0.0, make sure we produce a good version.
"""


import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestGetVersionForZero(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_get_version_zero(self):
        """Read in a library with a version of 0.0.0.  Test SBModule::GetVersion"""
        self.yaml2obj("libDylib.dylib.yaml", self.getBuildArtifact("libDylib.dylib"))
        self.do_test()

    def do_test(self):
        lib_name = "libDylib.dylib"
        target = lldbutil.run_to_breakpoint_make_target(self, exe_name=lib_name)
        module = target.FindModule(lldb.SBFileSpec(lib_name))
        self.assertTrue(module.IsValid(), "Didn't find the libDylib.dylib module")
        # For now the actual version numbers are wrong for a library of 0.0.0
        # but the previous code would crash iterating over the resultant
        # list.  So we are testing that that doesn't happen.
        did_iterate = False
        for elem in module.GetVersion():
            did_iterate = True
        self.assertTrue(did_iterate, "Didn't get into the GetVersion loop")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        """Test that SBType.GetBasicType unwraps typedefs."""
        self.build()
        lldbutil.run_to_source_breakpoint(self, "return", lldb.SBFileSpec("main.cpp"))

        a = self.frame().FindVariable("a")
        self.assertTrue(a)

        int_basic_type = a.GetType().GetBasicType()
        self.assertEqual(int_basic_type, 13)

        b = self.frame().FindVariable("b")
        self.assertTrue(b)

        c = self.frame().FindVariable("c")
        self.assertTrue(c)

        d = self.frame().FindVariable("d")
        self.assertTrue(d)

        self.assertEqual(b.GetType().GetBasicType(), int_basic_type)
        self.assertEqual(c.GetType().GetBasicType(), int_basic_type)
        self.assertEqual(d.GetType().GetBasicType(), int_basic_type)

        # Check the size of the chosen basic types.
        self.assertEqual(self.target().FindFirstType("__int128").size, 16)
        self.assertEqual(self.target().FindFirstType("unsigned __int128").size, 16)

        # Check the size of the chosen aliases of basic types.
        self.assertEqual(self.target().FindFirstType("__int128_t").size, 16)
        self.assertEqual(self.target().FindFirstType("__uint128_t").size, 16)

        self.assertFalse(self.target().FindFirstType("_BitInt"))
        self.assertFalse(self.target().FindFirstType("unsigned _BitInt"))
        self.assertFalse(self.target().FindFirstType("_BitInt()"))
        self.assertFalse(self.target().FindFirstType("unsigned _BitInt()"))
        self.assertFalse(self.target().FindFirstType("_BitInt(65"))
        self.assertFalse(self.target().FindFirstType("unsigned _BitInt(65"))
        self.assertFalse(self.target().FindFirstType("_BitInt(0x41)"))
        self.assertFalse(self.target().FindFirstType("unsigned _BitInt(0x41)"))
        self.assertEqual(self.target().FindFirstType("_BitInt(65)").name, "_BitInt(65)")
        self.assertEqual(self.target().FindFirstType("_BitInt(65)").size, 16)
        self.assertEqual(
            self.target().FindFirstType("unsigned _BitInt(65)").name,
            "unsigned _BitInt(65)",
        )
        self.assertEqual(self.target().FindFirstType("unsigned _BitInt(65)").size, 16)



# RUN: %PYTHON %s | FileCheck %s

from mlir import ir
from mlir.dialects import transform, smt
from mlir.dialects.transform import smt as transform_smt


def run(f):
    print("\nTEST:", f.__name__)
    with ir.Context(), ir.Location.unknown():
        module = ir.Module.create()
        with ir.InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with ir.InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print(module)
    return f


# CHECK-LABEL: TEST: testConstrainParamsOp
@run
def testConstrainParamsOp(target):
    c42_attr = ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 42)
    # CHECK: %[[PARAM_AS_PARAM:.*]] = transform.param.constant
    symbolic_value_as_param = transform.ParamConstantOp(
        transform.AnyParamType.get(), c42_attr
    )
    # CHECK: transform.smt.constrain_params(%[[PARAM_AS_PARAM]])
    constrain_params = transform_smt.ConstrainParamsOp(
        [], [symbolic_value_as_param], [smt.IntType.get()]
    )
    # CHECK-NEXT: ^bb{{.*}}(%[[PARAM_AS_SMT_SYMB:.*]]: !smt.int):
    with ir.InsertionPoint(constrain_params.body):
        symbolic_value_as_smt_var = constrain_params.body.arguments[0]
        # CHECK: %[[C0:.*]] = smt.int.constant 0
        c0 = smt.IntConstantOp(ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 0))
        # CHECK: %[[C43:.*]] = smt.int.constant 43
        c43 = smt.IntConstantOp(ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 43))
        # CHECK: %[[LB:.*]] = smt.int.cmp le %[[C0]], %[[PARAM_AS_SMT_SYMB]]
        lb = smt.IntCmpOp(smt.IntPredicate.le, c0, symbolic_value_as_smt_var)
        # CHECK: %[[UB:.*]] = smt.int.cmp le %[[PARAM_AS_SMT_SYMB]], %[[C43]]
        ub = smt.IntCmpOp(smt.IntPredicate.le, symbolic_value_as_smt_var, c43)
        # CHECK: %[[BOUNDED:.*]] = smt.and %[[LB]], %[[UB]]
        bounded = smt.AndOp([lb, ub])
        # CHECK: smt.assert %[[BOUNDED:.*]]
        smt.AssertOp(bounded)
        smt.YieldOp([])

    # CHECK: transform.smt.constrain_params(%[[PARAM_AS_PARAM]])
    compute_with_params = transform_smt.ConstrainParamsOp(
        [transform.AnyParamType.get()],
        [symbolic_value_as_param],
        [smt.IntType.get()],
    )
    # CHECK-NEXT: ^bb{{.*}}(%[[SMT_SYMB:.*]]: !smt.int):
    with ir.InsertionPoint(compute_with_params.body):
        symbolic_value_as_smt_var = compute_with_params.body.arguments[0]
        # CHECK: %[[TWICE:.*]] = smt.int.add %[[SMT_SYMB]], %[[SMT_SYMB]]
        twice_symb = smt.IntAddOp(
            [symbolic_value_as_smt_var, symbolic_value_as_smt_var]
        )
        # CHECK: smt.yield %[[TWICE]]
        smt.YieldOp([twice_symb])



# RUN: %PYTHON -m mlir.dialects.linalg.opdsl.dump_oplib --file %s | FileCheck %s

from mlir.dialects.linalg.opdsl.lang import *


# Verify that simple case with iteration order defined lexically and reduction
# dims auto discovered emits the right shape, indexing maps and iterator types.
# CHECK: ---
# CHECK-LABEL: matmul
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s0, s1)>
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s1, s2)>
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s0, s2)>
# CHECK: static_indexing_maps:
# CHECK-NEXT: - affine_map<(d0, d1, d2)[s0, s1, s2] -> (d0, d2)>
# CHECK-NEXT: - affine_map<(d0, d1, d2)[s0, s1, s2] -> (d2, d1)>
# CHECK-NEXT: - affine_map<(d0, d1, d2)[s0, s1, s2] -> (d0, d1)>
# CHECK: iterator_types:
# CHECK-NEXT: - parallel
# CHECK-NEXT: - parallel
# CHECK-NEXT: - reduction
@linalg_structured_op
def matmul(
    A=TensorDef(T, S.M, S.K),
    B=TensorDef(T, S.K, S.N),
    C=TensorDef(U, S.M, S.N, output=True),
):
    domain(D.m, D.n, D.k)
    C[D.m, D.n] += TypeFn.cast_signed(U, A[D.m, D.k]) * TypeFn.cast_signed(
        U, B[D.k, D.n]
    )


# Verifies that assignment to a scalar (represented as [None]) is represented
# correctly.
# CHECK: ---
# CHECK-LABEL: dot
# CHECK: shape_map: affine_map<()[s0] -> (s0)>
# CHECK: shape_map: affine_map<()[s0] -> (s0)>
# CHECK: shape_map: affine_map<()[s0] -> ()>
# CHECK: static_indexing_maps:
# CHECK-NEXT: - affine_map<(d0)[s0] -> (d0)>
# CHECK-NEXT: - affine_map<(d0)[s0] -> (d0)>
# CHECK-NEXT: - affine_map<(d0)[s0] -> ()>
# CHECK: iterator_types:
# CHECK-NEXT: - reduction
@linalg_structured_op
def dot(A=TensorDef(T, S.M), B=TensorDef(T, S.M), C=TensorDef(U, output=True)):
    C[None] += TypeFn.cast_signed(U, A[D.m]) * TypeFn.cast_signed(U, B[D.m])


# Verifies that the index_dims of shape-only operands translate to correct
# indexing maps.
# CHECK: ---
# CHECK-LABEL: pool
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s0)>
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s1)>
# CHECK: shape_map: affine_map<()[s0, s1, s2] -> (s2)>
# CHECK: static_indexing_maps:
# CHECK-NEXT: - affine_map<(d0, d1)[s0, s1, s2] -> (d0 * 2 + d1)>
# CHECK-NEXT: - affine_map<(d0, d1)[s0, s1, s2] -> (d1)>
# CHECK-NEXT: - affine_map<(d0, d1)[s0, s1, s2] -> (d0)>
# CHECK: iterator_types:
# CHECK-NEXT: - parallel
# CHECK-NEXT: - reduction
@linalg_structured_op
def pool(
    I=TensorDef(T, S.I),
    K=TensorDef(T, S.K, index_dims=[D.k]),
    O=TensorDef(U, S.O, output=True),
):
    domain(D.o, D.k)
    O[D.o] += TypeFn.cast_signed(U, I[D.o * 2 + D.k])



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import expectedFailureWindows
import lldbdap_testcase


class TestDAP_launch_basic(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the default launch of a simple program. No arguments,
    environment, or anything else is specified.
    """

    def test(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        self.continue_to_exit()
        # Now get the STDOUT and verify our program argument is correct
        output = self.get_stdout()
        self.assertTrue(output and len(output) > 0, "expect program output")
        lines = output.splitlines()
        self.assertIn(program, lines[0], "make sure program path is in first argument")



"""
Test 'frame select' command.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestFrameSelect(TestBase):
    @no_debug_info_test
    @skipIfWindows
    def test_relative(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.expect("frame select -r 1", substrs=["nested2() at"])
        self.expect("frame select -r -1", substrs=["nested3() at"])

        self.expect(
            "frame select -r -1",
            error=True,
            substrs=["already at the bottom of the stack"],
        )
        self.expect(
            "frame select -r -2147483647",
            error=True,
            substrs=["already at the bottom of the stack"],
        )
        self.expect(
            "frame select -r -2147483648",
            error=True,
            substrs=["error: invalid frame offset argument '-2147483648'"],
        )
        self.expect(
            "frame select -r -2147483649",
            error=True,
            substrs=["error: invalid frame offset argument '-2147483649'"],
        )

        self.expect("frame select -r 1", substrs=["nested2() at"])
        self.expect("frame select -r -2", substrs=["nested3() at"])
        self.expect("frame select -r 1", substrs=["nested2() at"])
        self.expect("frame select -r -2147483647", substrs=["nested3() at"])
        self.expect("frame select -r 1", substrs=["nested2() at"])
        self.expect(
            "frame select -r -2147483648",
            error=True,
            substrs=["error: invalid frame offset argument '-2147483648'"],
        )
        self.expect(
            "frame select -r -2147483649",
            error=True,
            substrs=["error: invalid frame offset argument '-2147483649'"],
        )

        self.expect("frame select -r 100")
        self.expect(
            "frame select -r 1",
            error=True,
            substrs=["already at the top of the stack"],
        )

    @no_debug_info_test
    @skipIfWindows
    def test_mixing_relative_and_abs(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # The function associated with each frame index can change depending
        # on the function calling main (e.g. `start`), so this only tests that
        # the frame index number is correct. We test the actual functions
        # in the relative test.

        # Jump to the top of the stack.
        self.expect("frame select 0", substrs=["frame #0"])

        # Run some relative commands.
        self.expect("up", substrs=["frame #1"])
        self.expect("frame select -r 1", substrs=["frame #2"])
        self.expect("frame select -r -1", substrs=["frame #1"])

        # Test that absolute indices still work.
        self.expect("frame select 2", substrs=["frame #2"])
        self.expect("frame select 1", substrs=["frame #1"])
        self.expect("frame select 3", substrs=["frame #3"])
        self.expect("frame select 0", substrs=["frame #0"])
        self.expect("frame select 1", substrs=["frame #1"])

        # Run some other relative frame select commands.
        self.expect("down", substrs=["frame #0"])
        self.expect("frame select -r 1", substrs=["frame #1"])
        self.expect("frame select -r -1", substrs=["frame #0"])

        # Test that absolute indices still work.
        self.expect("frame select 2", substrs=["frame #2"])
        self.expect("frame select 1", substrs=["frame #1"])
        self.expect("frame select 3", substrs=["frame #3"])
        self.expect("frame select 0", substrs=["frame #0"])



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import skipIf
from lldbsuite.test.lldbtest import line_number
import lldbdap_testcase


class TestDAP_launch_commands(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the "initCommands", "preRunCommands", "stopCommands",
    "terminateCommands" and "exitCommands" that can be passed during
    launch.

    "initCommands" are a list of LLDB commands that get executed
    before the target is created.
    "preRunCommands" are a list of LLDB commands that get executed
    after the target has been created and before the launch.
    "stopCommands" are a list of LLDB commands that get executed each
    time the program stops.
    "exitCommands" are a list of LLDB commands that get executed when
    the process exits
    "terminateCommands" are a list of LLDB commands that get executed when
    the debugger session terminates.
    """

    @skipIf(archs=["arm$", "aarch64"], bugnumber=6933)
    def test(self):
        program = self.getBuildArtifact("a.out")
        initCommands = ["target list", "platform list"]
        preRunCommands = ["image list a.out", "image dump sections a.out"]
        postRunCommands = ["help trace", "help process trace"]
        stopCommands = ["frame variable", "bt"]
        exitCommands = ["expr 2+3", "expr 3+4"]
        terminateCommands = ["expr 4+2"]
        self.build_and_launch(
            program,
            initCommands=initCommands,
            preRunCommands=preRunCommands,
            postRunCommands=postRunCommands,
            stopCommands=stopCommands,
            exitCommands=exitCommands,
            terminateCommands=terminateCommands,
        )
        self.dap_server.wait_for_initialized()

        # Get output from the console. This should contain both the
        # "initCommands" and the "preRunCommands".
        output = self.collect_console(pattern=postRunCommands[-1])
        # Verify all "initCommands" were found in console output
        self.verify_commands("initCommands", output, initCommands)
        # Verify all "preRunCommands" were found in console output
        self.verify_commands("preRunCommands", output, preRunCommands)
        # Verify all "postRunCommands" were found in console output
        self.verify_commands("postRunCommands", output, postRunCommands)

        source = "main.c"
        first_line = line_number(source, "// breakpoint 1")
        second_line = line_number(source, "// breakpoint 2")
        lines = [first_line, second_line]

        # Set 2 breakpoints so we can verify that "stopCommands" get run as the
        # breakpoints get hit
        breakpoint_ids = self.set_source_breakpoints(source, lines)
        self.assertEqual(
            len(breakpoint_ids), len(lines), "expect correct number of breakpoints"
        )

        # Continue after launch and hit the first breakpoint.
        # Get output from the console. This should contain both the
        # "stopCommands" that were run after the first breakpoint was hit
        self.continue_to_breakpoints(breakpoint_ids)
        output = self.collect_console(pattern=stopCommands[-1])
        self.verify_commands("stopCommands", output, stopCommands)

        # Continue again and hit the second breakpoint.
        # Get output from the console. This should contain both the
        # "stopCommands" that were run after the second breakpoint was hit
        self.continue_to_breakpoints(breakpoint_ids)
        output = self.collect_console(pattern=stopCommands[-1])
        self.verify_commands("stopCommands", output, stopCommands)

        # Continue until the program exits
        self.continue_to_exit()
        # Get output from the console. This should contain both the
        # "exitCommands" that were run after the second breakpoint was hit
        # and the "terminateCommands" due to the debugging session ending
        output = self.collect_console(pattern=terminateCommands[0])
        self.verify_commands("exitCommands", output, exitCommands)
        self.verify_commands("terminateCommands", output, terminateCommands)



# UNSUPPORTED: target=aarch64{{.*}}, target=arm64{{.*}}
# RUN: %PYTHON %s 2>&1 | FileCheck %s
# REQUIRES: host-supports-jit
import gc, sys, os, tempfile
from mlir.ir import *
from mlir.passmanager import *
from mlir.execution_engine import *
from mlir.runtime import *


# Log everything to stderr and flush so that we have a unified stream to match
# errors/info emitted by MLIR to stderr.
def log(*args):
    print(*args, file=sys.stderr)
    sys.stderr.flush()


def run(f):
    log("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0


def lowerToLLVM(module):
    pm = PassManager.parse(
        "builtin.module(convert-func-to-llvm,reconcile-unrealized-casts)"
    )
    pm.run(module.operation)
    return module


# Test JIT callback in global constructor
# CHECK-LABEL: TEST: testJITCallbackInGlobalCtor
def testJITCallbackInGlobalCtor():
    init_cnt = 0

    @ctypes.CFUNCTYPE(None)
    def initCallback():
        nonlocal init_cnt
        init_cnt += 1

    with Context():
        module = Module.parse(
            r"""
llvm.mlir.global_ctors ctors = [@ctor], priorities = [0 : i32], data = [#llvm.zero]
llvm.func @ctor() {
  func.call @init_callback() : () -> ()
  llvm.return
}
func.func private @init_callback() attributes { llvm.emit_c_interface }
        """
        )

        # Setup execution engine
        execution_engine = ExecutionEngine(lowerToLLVM(module))

        # Validate initialization hasn't run yet
        assert init_cnt == 0

        # # Register callback
        execution_engine.register_runtime("init_callback", initCallback)

        # # Initialize and verify
        execution_engine.initialize()
        assert init_cnt == 1
        # # Second initialization should be no-op
        execution_engine.initialize()
        assert init_cnt == 1


run(testJITCallbackInGlobalCtor)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestThreadInfoTrailingComma(GDBRemoteTestBase):
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def haltReason(self):
                return "T02thread:1"

            def qfThreadInfo(self):
                return "m1,2,3,4,"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        process = self.connect(target)
        self.assertEqual(process.GetThreadAtIndex(0).GetThreadID(), 1)
        self.assertEqual(process.GetThreadAtIndex(1).GetThreadID(), 2)
        self.assertEqual(process.GetThreadAtIndex(2).GetThreadID(), 3)
        self.assertEqual(process.GetThreadAtIndex(3).GetThreadID(), 4)



"""
Test that we properly print multiple types.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test import decorators


class TestTypeLookupDuplicate(TestBase):
    def test_namespace_only(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.expect(
            "image lookup -A -t Foo",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=["2 matches found", "\nid =", "\nid ="],
        )



"""
Make sure that we can step in across an arm64 branch island
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestBranchIslandStepping(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessAppleSilicon
    def test_step_in_branch_island(self):
        """Make sure we can step in across a branch island"""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.do_test()

    def do_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        # Make sure that we did manage to generate a branch island for foo:
        syms = target.FindSymbols("foo.island", lldb.eSymbolTypeCode)
        self.assertEqual(len(syms), 1, "We did generate an island for foo")
        # There's a bug in the Xcode 15.2 linker, where it did not emit
        # "function starts" entries for the branch island symbols, which
        # causes lldb to set the range of the previous non-island symbol to
        # encompass the range of the branch island symbols.  If we encounter
        # that bug, then we won't successfully do the step in.  Test for
        # that here - if the symbol doesn't round-trip from
        # name->address->name then the rest of the test can't pass.
        island_sym_ctx = syms[0]
        sym_addr = island_sym_ctx.symbol.addr
        resolved_name = sym_addr.symbol.name
        if resolved_name != "foo.island":
            self.skipTest("Encountered overlapping symbol linker bug")
        thread.StepInto()
        stop_frame = thread.frames[0]
        self.assertIn("foo", stop_frame.name, "Stepped into foo")
        var = stop_frame.FindVariable("a_variable_in_foo")
        self.assertTrue(var.IsValid(), "Found the variable in foo")



# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from pygments.lexer import RegexLexer, bygroups, include, using
from pygments.token import *
import re

__all__ = ["MlirLexer"]


class MlirLexer(RegexLexer):
    """Pygments lexer for MLIR.

    This lexer focuses on accurate tokenization of common MLIR constructs:
    - SSA values (%%... / %...)
    - attribute and type aliases (#name =, !name =)
    - types (builtin and dialect types, parametric types)
    - attribute dictionaries and nested containers to a reasonable depth
    - numbers (ints, floats with exponents, hex)
    - strings with common escapes
    - line comments (// ...)
    - block labels (^foo) and operations
    """

    name = "MLIR"
    aliases = ["mlir"]
    filenames = ["*.mlir"]

    flags = re.MULTILINE

    class VariableList(RegexLexer):
        """Lexer for lists of SSA variables separated by commas."""

        tokens = {
            "root": [
                (r"\s+", Text),
                (r",", Punctuation),
                (r"%[_A-Za-z0-9\.\$\-:#]+", Name.Variable),
            ]
        }

    tokens = {
        "root": [
            # Comments
            (r"//.*?$", Comment.Single),
            # operation name with assignment: %... = op.name
            (
                r"^(\s*)(%[\%_A-Za-z0-9\:#\,\s]+)(=)(\s*)([A-Za-z0-9_\.\$\-]+)\b",
                bygroups(Text, using(VariableList), Operator, Text, Name.Builtin),
            ),
            # operation name without result
            (r"^(\s*)([A-Za-z0-9_\.\$\-]+)\b(?=[^<:])", bygroups(Text, Name.Builtin)),
            # Attribute alias definition:  #name =
            (
                r"^(\s*)(#[_A-Za-z0-9\$\-\.]+)(\b)(\s*=)",
                bygroups(Text, Name.Constant, Text, Operator),
            ),
            # Type alias definition: !name =
            (
                r"^(\s*)(![_A-Za-z0-9\$\-\.]+)(\b)(\s*=)",
                bygroups(Text, Keyword.Type, Text, Operator),
            ),
            # SSA values (uses)
            (r"%[_A-Za-z0-9\.\$\-:#]+", Name.Variable),
            # attribute refs, constants and named attributes
            (r"#[_A-Za-z0-9\$\-\.]+\b", Name.Constant),
            # symbol refs / function-like names
            (r"@[_A-Za-z][_A-Za-z0-9\$\-\.]*\b", Name.Function),
            # blocks
            (r"\^[A-Za-z0-9_\$\.\-]+", Name.Label),
            # types by exclamation or builtin names
            (r"![_A-Za-z0-9\$\-\.]+\b", Keyword.Type),
            # NOTE: please sync changes to corresponding builtin type rule in "angled-type"
            (r"\b(bf16|f16|f32|f64|f80|f128|index|none|(u|s)?i[0-9]+)\b", Keyword.Type),
            # container-like dialect types (tensor<...>, memref<...>, vector<...>)
            (
                r"\b(complex|memref|tensor|tuple|vector)\s*(<)",
                bygroups(Keyword.Type, Punctuation),
                "angled-type",
            ),
            # affine constructs
            (r"\b(affine_map|affine_set)\b", Keyword.Reserved),
            # common builtin operators / functions inside affine_map
            (r"\b(ceildiv|floordiv|mod|symbol)\b", Name.Other),
            # identifiers / bare words
            (r"\b[_A-Za-z][_A-Za-z0-9\.-]*\b", Name.Other),
            # numbers: hex, float (with exponent), integer
            (r"\b0x[0-9A-Fa-f]+\b", Number.Hex),
            (r"\b([0-9]+(\.[0-9]*)?|\.[0-9]+)([eE][+-]?[0-9]+)?\b", Number.Float),
            (r"\b[0-9]+\b", Number.Integer),
            # strings
            (r'"', String.Double, "string"),
            # punctuation and arrow-like tokens
            (r"->|>=|<=|\>=|\<=|\->|\=>", Operator),
            (r"[()\[\]{}<>,.:=]", Punctuation),
            # operators
            (r"[-+*/%]", Operator),
        ],
        # string state with common escapes
        "string": [
            (r'\\[ntr"\\]', String.Escape),
            (r'[^"\\]+', String.Double),
            (r'"', String.Double, "#pop"),
        ],
        # angled-type content
        "angled-type": [
            # match nested '<' and '>'
            (r"<", Punctuation, "#push"),
            (r">", Punctuation, "#pop"),
            # dimensions like 3x or 3x3x... and standalone numbers:
            # - match numbers that are followed by an 'x' (dimension separator)
            (r"([0-9]+)(?=(?:x))", Number.Integer),
            # - match bare numbers (sizes)
            (r"[0-9]+", Number.Integer),
            # dynamic dimension '?'
            (r"\?", Name.Integer),
            # the 'x' dimension separator (treat as punctuation)
            (r"x", Punctuation),
            # element / builtin types inside angle brackets (no word-boundary)
            # NOTE: please sync changes to corresponding builtin type rule in "root"
            (
                r"(?:bf16|f16|f32|f64|f80|f128|index|none|(?:[us]?i[0-9]+))",
                Keyword.Type,
            ),
            # also allow nested container-like types to be recognized
            (
                r"\b(complex|memref|tensor|tuple|vector)\s*(<)",
                bygroups(Keyword.Type, Punctuation),
                "angled-type",
            ),
            # fall back to root rules for anything else
            include("root"),
        ],
    }



# RUN: env PYTHONUNBUFFERED=1 %PYTHON %s 2>&1 | FileCheck %s

from typing import Sequence

from contextlib import contextmanager

from mlir import ir
from mlir.dialects import index, transform, func, arith, ext
from mlir.dialects.transform import (
    DiagnosedSilenceableFailure,
    AnyOpType,
    AnyValueType,
    AnyParamType,
    structured,
    interpreter,
)


class MyTransform(ext.Dialect, name="my_transform"):
    pass


def run(emit_schedule):
    print(f"Test: {emit_schedule.__name__}")
    with ir.Context() as ctx, ir.Location.unknown():
        payload = emit_payload()

        MyTransform.load(reload=True)

        GetNamedAttributeOp.attach_interface_impls(ctx)
        PrintParamOp.attach_interface_impls(ctx)

        # NB: Other newly defined my_transform ops have their interfaces attached
        #     in their respective test functions.
        schedule = emit_schedule()

        interpreter.apply_named_sequence(
            payload,
            _named_seq := schedule.operation.regions[0].blocks[0].operations[0],
            schedule,
        )


# Payload used by all tests
def emit_payload():
    payload_module = ir.Module.create()
    with ir.InsertionPoint(payload_module.body):
        f32 = ir.F32Type.get()

        @func.FuncOp.from_py_func(f32, f32, results=[f32])
        def name_of_func(a, b):
            c = arith.addf(a, b)
            i32 = ir.IntegerType.get_signless(32)
            arith.constant(i32, 42)
            arith.constant(i32, 24)
            func.ReturnOp([c])

    return payload_module


@contextmanager
def schedule_boilerplate():
    schedule = ir.Module.create()
    schedule.operation.attributes["transform.with_named_sequence"] = ir.UnitAttr.get()
    with ir.InsertionPoint(schedule.body):
        named_sequence = transform.NamedSequenceOp(
            "__transform_main",
            [AnyOpType.get()],
            [AnyOpType.get()],
            arg_attrs=[{"transform.consumed": ir.UnitAttr.get()}],
        )
        with ir.InsertionPoint(named_sequence.body):
            yield schedule, named_sequence


# MemoryEffectsOpInterface implementation for TransformOpInterface-implementing ops.
# Used by most ops defined below.
class MemoryEffectsOpInterfaceFallbackModel(ir.MemoryEffectsOpInterface):
    @staticmethod
    def get_effects(op: ir.Operation, effects):
        transform.only_reads_handle(op.op_operands, effects)
        transform.produces_handle(op.results, effects)
        transform.only_reads_payload(effects)


# Demonstration of a TransformOpInterface-implementing op that gets named attributes
# from target ops and produces them as param handles.
class GetNamedAttributeOp(MyTransform.Operation, name="get_named_attribute"):
    target: ext.Operand[transform.AnyOpType]
    attr_name: ir.StringAttr
    attr_as_param: ext.Result[transform.AnyParamType[()]] = ext.result(infer_type=True)

    @classmethod
    def attach_interface_impls(cls, ctx=None):
        cls.TransformOpInterfaceFallbackModel.attach(cls.OPERATION_NAME, context=ctx)
        MemoryEffectsOpInterfaceFallbackModel.attach(cls.OPERATION_NAME, context=ctx)

    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: "GetNamedAttributeOp",
            _rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            target_ops = state.get_payload_ops(op.target)
            associated_attrs = []
            for target_op in target_ops:
                assoc_attr = target_op.attributes.get(op.attr_name.value)
                if assoc_attr is None:
                    return DiagnosedSilenceableFailure.RecoverableFailure
                associated_attrs.append(assoc_attr)
            results.set_params(op.attr_as_param, associated_attrs)
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: "GetNamedAttributeOp") -> bool:
            return False


class PrintParamOp(MyTransform.Operation, name="print_param"):
    target: ext.Operand[transform.AnyParamType]
    name: ir.StringAttr

    @classmethod
    def attach_interface_impls(cls, ctx=None):
        cls.TransformOpInterfaceFallbackModel.attach(cls.OPERATION_NAME, context=ctx)
        MemoryEffectsOpInterfaceFallbackModel.attach(cls.OPERATION_NAME, context=ctx)

    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: "PrintParamOp",
            rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            target_attrs = state.get_params(op.target)
            print(f"[[[ IR printer: {op.name.value} ]]]")
            for attr in target_attrs:
                print(attr)
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: "GetNamedAttributeOp") -> bool:
            return False


# Syntax for an op with one op handle operand and one op handle result.
class OneOpInOneOpOut(MyTransform.Operation, name="one_op_in_one_op_out"):
    target: ext.Operand[transform.AnyOpType]
    res: ext.Result[transform.AnyOpType[()]] = ext.result(infer_type=True)


# CHECK-LABEL: Test: OneOpInOneOpOutTransformOpInterface
@run
def OneOpInOneOpOutTransformOpInterface():
    """Tests a simple passthrough interface implementation.

    Checks that the target ops are correctly identified and passed as results.
    """

    # Define a simple passthrough implementation of the TransformOpInterface for OneOpInOneOpOut.
    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: OneOpInOneOpOut,
            _rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            target_ops = state.get_payload_ops(op.target)
            target_names = [t.name.value for t in target_ops]
            print(f"OneOpInOneOpOutTransformOpInterface: target_names={target_names}")
            results.set_ops(op.res, target_ops)
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: OneOpInOneOpOut) -> bool:
            return False

    # Attach the interface implementation to the op.
    TransformOpInterfaceFallbackModel.attach(OneOpInOneOpOut.OPERATION_NAME)

    # TransformOpInterface-implementing ops are also required to implement MemoryEffectsOpInterface. The above defined fallback model works for this op.
    MemoryEffectsOpInterfaceFallbackModel.attach(OneOpInOneOpOut.OPERATION_NAME)

    with schedule_boilerplate() as (schedule, named_seq):
        func_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["func.func"]
        ).result
        # CHECK: OneOpInOneOpOutTransformOpInterface: target_names=['name_of_func']
        out = OneOpInOneOpOut(func_handle).result
        # CHECK: Output handle from OneOpInOneOpOut
        # CHECK-NEXT: func.func @name_of_func
        transform.PrintOp(target=out, name="Output handle from OneOpInOneOpOut")
        transform.YieldOp([out])

    return schedule


# CHECK-LABEL: Test: OneOpInOneOpOutTransformOpInterfaceRewriterImpl
@run
def OneOpInOneOpOutTransformOpInterfaceRewriterImpl():
    """Tests an interface implementation using the rewriter to modify the IR.

    Checks that `arith.constant` ops are replaced by `index.constant` ops and
    that the results are correctly updated.
    """

    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: OneOpInOneOpOut,
            rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            result_ops = []
            for target_op in state.get_payload_ops(op.target):
                with ir.InsertionPoint(target_op):
                    index_version = index.constant(target_op.value.value)
                result_ops.append(index_version.owner)
                rewriter.replace_op(target_op, [index_version])
            results.set_ops(op.res, result_ops)
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: OneOpInOneOpOut) -> bool:
            return False

    # Attach the interface implementation to the op.
    TransformOpInterfaceFallbackModel.attach(OneOpInOneOpOut.OPERATION_NAME)

    # TransformOpInterface-implementing ops are also required to implement MemoryEffectsOpInterface. The above defined fallback model works for this op.
    class MemoryEffectsOpInterfaceFallbackModel(ir.MemoryEffectsOpInterface):
        @staticmethod
        def get_effects(op: ir.Operation, effects):
            transform.consumes_handle(op.op_operands, effects)
            transform.produces_handle(op.results, effects)
            transform.modifies_payload(effects)

    MemoryEffectsOpInterfaceFallbackModel.attach(OneOpInOneOpOut.OPERATION_NAME)

    with schedule_boilerplate() as (schedule, named_seq):
        func_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["func.func"]
        ).result
        csts_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["arith.constant"]
        ).result
        # CHECK: Before replacement:
        # CHECK-NOT: index.constant
        # CHECK-DAG: arith.constant 42 : i32
        # CHECK-DAG: arith.constant 24 : i32
        transform.PrintOp(target=func_handle, name="Before replacement:")
        out = OneOpInOneOpOut(csts_handle).result
        # CHECK: After replacement:
        # CHECK-NOT: arith.constant
        # CHECK-DAG: index.constant 42
        # CHECK-DAG: index.constant 24
        transform.PrintOp(target=func_handle, name="After replacement:")
        # CHECK: Output handle from OneOpInOneOpOut:
        # CHECK-NEXT: index.constant 42
        # CHECK-NEXT: index.constant 24
        transform.PrintOp(target=out, name="Output handle from OneOpInOneOpOut:")
        transform.YieldOp([out])

    return schedule


class OpValParamInParamOpValOut(
    MyTransform.Operation, name="op_val_param_in_param_op_val_out"
):
    # operands
    op_arg: ext.Operand[transform.AnyOpType]
    val_arg: ext.Operand[transform.AnyValueType]
    param_arg: ext.Operand[transform.AnyParamType]
    # results
    param_res: ext.Result[transform.AnyParamType[()]] = ext.result(infer_type=True)
    op_res: ext.Result[transform.AnyOpType[()]] = ext.result(infer_type=True)
    value_res: ext.Result[transform.AnyValueType[()]] = ext.result(infer_type=True)


# CHECK-LABEL: Test: OpValParamInParamOpValOutTransformOpInterface
@run
def OpValParamInParamOpValOutTransformOpInterface():
    """Tests an interface implementation involving Op, Value, and Param types.

    Checks that payload ops, values, and parameters are correctly permuted and
    propagated and accessible from the (permuted) result handles.
    """

    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: OpValParamInParamOpValOut,
            _rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            ops = state.get_payload_ops(op.op_arg)
            values = state.get_payload_values(op.val_arg)
            params = state.get_params(op.param_arg)
            print(
                f"OpValParamInParamOpValOutTransformOpInterface: ops={len(ops)}, values={len(values)}, params={len(params)}"
            )
            results.set_params(op.param_res, params)
            results.set_ops(op.op_res, ops)
            results.set_values(op.value_res, values)
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: OpValParamInParamOpValOut) -> bool:
            return False

    TransformOpInterfaceFallbackModel.attach(OpValParamInParamOpValOut.OPERATION_NAME)

    # TransformOpInterface-implementing ops are also required to implement MemoryEffectsOpInterface. The above defined fallback model works for this op.
    MemoryEffectsOpInterfaceFallbackModel.attach(
        OpValParamInParamOpValOut.OPERATION_NAME
    )

    with schedule_boilerplate() as (schedule, named_seq):
        func_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["func.func"]
        ).result
        addf_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["arith.addf"]
        ).result
        func_and_addf = transform.MergeHandlesOp([func_handle, addf_handle])
        value_handle = transform.GetResultOp(
            AnyValueType.get(), addf_handle, [0]
        ).result
        param_handle = transform.ParamConstantOp(
            AnyParamType.get(), ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 42)
        ).param

        # CHECK: OpValParamInParamOpValOutTransformOpInterface: ops=2, values=1, params=1
        op_val_param_op = OpValParamInParamOpValOut(
            func_and_addf, value_handle, param_handle
        )
        # CHECK: Ops passed through OpValParamInParamOpValOut:
        # CHECK-NEXT: func.func
        # CHECK: arith.addf
        transform.PrintOp(
            target=op_val_param_op.op_res,
            name="Ops passed through OpValParamInParamOpValOut:",
        )

        # CHECK: Ops defining values passed through OpValParamInParamOpValOut:
        # CHECK-NEXT: arith.addf
        addf_as_res = transform.GetDefiningOp(
            transform.AnyOpType.get(), op_val_param_op.value_res
        ).result
        transform.PrintOp(
            target=addf_as_res,
            name="Ops defining values passed through OpValParamInParamOpValOut:",
        )

        # CHECK: Parameter passed through OpValParamInParamOpValOut:
        # CHECK-NEXT: 42 : i32
        PrintParamOp(
            op_val_param_op.param_res,
            name=ir.StringAttr.get(
                "Parameter passed through OpValParamInParamOpValOut:"
            ),
        )

        transform.YieldOp([op_val_param_op.op_res])
        named_seq.verify()

    return schedule


class OpsParamsInValuesParamOut(
    MyTransform.Operation, name="ops_params_in_values_param_out"
):
    # results
    values: Sequence[ext.Result[transform.AnyValueType]]
    param: ext.Result[transform.AnyParamType]
    # operands
    ops: Sequence[ext.Operand[transform.AnyOpType]]
    params: Sequence[ext.Operand[transform.AnyParamType]]


# CHECK-LABEL: Test: OpsParamsInValuesParamOutTransformOpInterface
@run
def OpsParamsInValuesParamOutTransformOpInterface():
    """Tests an interface with variadic Op and Param operands and variadic Value results.

    Checks correct handling of multiple handles, parameter aggregation, and
    result generation.
    """

    class TransformOpInterfaceFallbackModel(transform.TransformOpInterface):
        @staticmethod
        def apply(
            op: OpsParamsInValuesParamOut,
            _rewriter: transform.TransformRewriter,
            results: transform.TransformResults,
            state: transform.TransformState,
        ) -> DiagnosedSilenceableFailure:
            ops_count = 0
            value_handles = []
            for op_handle in op.ops:
                ops = state.get_payload_ops(op_handle)
                ops_count += len(ops)
                value_handles.append([i for op in ops for i in op.results])

            param_count = 0
            param_sum = 0
            for param_handle in op.params:
                params = state.get_params(param_handle)
                param_count += len(params)
                param_sum += sum(p.value for p in params)

            print(
                f"OpsParamsInValuesParamOutTransformOpInterfaceFallbackModel: op_count={ops_count}, param_count={param_count}"
            )

            assert len(op.values) == len(op.ops)
            for value_res_handle, value_vector in zip(op.values, value_handles):
                results.set_values(value_res_handle, value_vector)
            results.set_params(
                op.param,
                [ir.IntegerAttr.get(ir.IntegerType.get_signless(32), param_sum)],
            )
            return DiagnosedSilenceableFailure.Success

        @staticmethod
        def allow_repeated_handle_operands(_op: OpsParamsInValuesParamOut) -> bool:
            return False

    TransformOpInterfaceFallbackModel.attach(OpsParamsInValuesParamOut.OPERATION_NAME)

    MemoryEffectsOpInterfaceFallbackModel.attach(
        OpsParamsInValuesParamOut.OPERATION_NAME
    )

    with schedule_boilerplate() as (schedule, named_seq):
        func_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["func.func"]
        ).result
        csts_handle = structured.MatchOp.match_op_names(
            named_seq.bodyTarget, ["arith.constant"]
        ).result
        csts_as_param = GetNamedAttributeOp(
            csts_handle, attr_name=ir.StringAttr.get("value")
        ).attr_as_param

        param_handle = transform.ParamConstantOp(
            AnyParamType.get(), ir.IntegerAttr.get(ir.IntegerType.get_signless(32), 123)
        ).param

        # CHECK: OpsParamsInValuesParamOutTransformOpInterfaceFallbackModel: op_count=3, param_count=3
        op = OpsParamsInValuesParamOut(
            [transform.AnyValueType.get()] * 2,
            transform.AnyParamType.get(),
            [func_handle, csts_handle],
            [csts_as_param, param_handle],
        )

        empty_handle = transform.GetDefiningOp(transform.AnyOpType.get(), op.values[0])
        # CHECK: Defining op of value result 0
        transform.PrintOp(
            target=empty_handle.result, name="Defining op of value result 0"
        )
        # NB: no result on the func.func, so output is expected to be empty
        cst1_res, cst2_res = transform.SplitHandleOp(
            [transform.AnyValueType.get()] * 2, op.values[1]
        ).results

        cst1_again = transform.GetDefiningOp(transform.AnyOpType.get(), cst1_res)
        # CHECK-NEXT: Defining op of first constant
        # CHECK-NEXT: arith.constant 42 : i32
        transform.PrintOp(
            target=cst1_again.result, name="Defining op of first constant"
        )
        cst2_again = transform.GetDefiningOp(transform.AnyOpType.get(), cst2_res)
        # CHECK-NEXT: Defining op of second constant
        # CHECK-NEXT: arith.constant 24 : i32
        transform.PrintOp(
            target=cst2_again.result, name="Defining op of second constant"
        )

        # CHECK: Sum of params:
        # CHECK-NEXT: 189 : i32
        PrintParamOp(op.param, name=ir.StringAttr.get("Sum of params:"))

        transform.YieldOp([func_handle])
        named_seq.verify()

    return schedule



"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class VarInAggregateMisuseTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test that that file and class static variables display correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type summary clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd(
            'type summary add --summary-string "SUMMARY SUCCESS ${var}" Summarize'
        )

        self.expect(
            "frame variable mine_ptr", substrs=["SUMMARY SUCCESS summarize_ptr_t @ "]
        )

        self.expect(
            "frame variable *mine_ptr", substrs=["SUMMARY SUCCESS summarize_t @"]
        )

        self.runCmd(
            'type summary add --summary-string "SUMMARY SUCCESS ${var.first}" Summarize'
        )

        self.expect("frame variable mine_ptr", substrs=["SUMMARY SUCCESS 10"])

        self.expect("frame variable *mine_ptr", substrs=["SUMMARY SUCCESS 10"])

        self.runCmd('type summary add --summary-string "${var}" Summarize')
        self.runCmd('type summary add --summary-string "${var}" -e TwoSummarizes')

        self.expect(
            "frame variable",
            substrs=[
                "(TwoSummarizes) twos = TwoSummarizes @ ",
                "first = summarize_t @ ",
                "second = summarize_t @ ",
            ],
        )

        self.runCmd(
            'type summary add --summary-string "SUMMARY SUCCESS ${var.first}" Summarize'
        )
        self.expect(
            "frame variable",
            substrs=[
                "(TwoSummarizes) twos = TwoSummarizes @ ",
                "first = SUMMARY SUCCESS 1",
                "second = SUMMARY SUCCESS 3",
            ],
        )



"""
Test that we are able to broadcast and receive diagnostic events from lldb
"""
import lldb

import lldbsuite.test.lldbutil as lldbutil

from lldbsuite.test.lldbtest import *


class TestDiagnosticReporting(TestBase):
    def setUp(self):
        TestBase.setUp(self)

        self.broadcaster = self.dbg.GetBroadcaster()
        self.listener = lldbutil.start_listening_from(
            self.broadcaster,
            lldb.SBDebugger.eBroadcastBitWarning | lldb.SBDebugger.eBroadcastBitError,
        )

    def test_dwarf_symbol_loading_diagnostic_report(self):
        """Test that we are able to fetch diagnostic events"""

        self.yaml2obj("minidump.yaml", self.getBuildArtifact("minidump.core"))

        self.dbg.CreateTarget(None)
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore(self.getBuildArtifact("minidump.core"))

        event = lldbutil.fetch_next_event(self, self.listener, self.broadcaster)
        diagnostic_data = lldb.SBDebugger.GetDiagnosticFromEvent(event)
        self.assertEqual(
            diagnostic_data.GetValueForKey("type").GetStringValue(100), "warning"
        )
        self.assertEqual(
            diagnostic_data.GetValueForKey("message").GetStringValue(100),
            "unable to retrieve process ID from minidump file, setting process ID to 1",
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_FindTypes_on_scratch_AST(self):
        """
        Tests FindTypes invoked with only LLDB's scratch AST present.
        """
        target = self.dbg.GetDummyTarget()
        # There should be only one instance of 'unsigned long' in our single
        # scratch AST. Note: FindTypes/SBType hahave no filter by language, so
        # pick something that is unlikely to also be found in the scratch
        # TypeSystem of other language plugins.
        self.assertEqual(len(target.FindTypes("unsigned long")), 1)



# encoding: utf-8
"""
Test lldb data formatter subsystem.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def appkit_tester_impl(self, commands, use_constant_classes):
        if use_constant_classes:
            self.build()
        else:
            disable_constant_classes = {
                "CFLAGS_EXTRAS": "-fno-constant-nsnumber-literals "
                + "-fno-constant-nsarray-literals "
                + "-fno-constant-nsdictionary-literals",
            }
            # FIXME: Remove compiler when flags are available upstream.
            self.build(dictionary=disable_constant_classes, compiler="xcrun clang")
        self.appkit_common_data_formatters_command()
        commands()

    def appkit_common_data_formatters_command(self):
        """Test formatters for AppKit classes."""
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.m", False)
        )

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type synth clear", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)



"""
Test the lldb disassemble command on foundation framework.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationDisassembleTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfAsan
    def test_simple_disasm(self):
        """Test the lldb 'disassemble' command"""
        self.build()

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        # Stop at +[NSString stringWithFormat:].
        symbol_name = "+[NSString stringWithFormat:]"
        break_results = lldbutil.run_break_set_command(
            self, "_regexp-break %s" % (symbol_name)
        )

        lldbutil.check_breakpoint_result(
            self, break_results, symbol_name=symbol_name, num_locations=1
        )

        # Stop at -[MyString initWithNSString:].
        lldbutil.run_break_set_by_symbol(
            self,
            "-[MyString initWithNSString:]",
            num_expected_locations=1,
            sym_exact=True,
        )

        # Stop at the "description" selector.
        lldbutil.run_break_set_by_selector(
            self, "description", num_expected_locations=1, module_name="a.out"
        )

        # Stop at -[NSAutoreleasePool release].
        break_results = lldbutil.run_break_set_command(
            self, "_regexp-break -[NSAutoreleasePool release]"
        )
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            symbol_name="-[NSAutoreleasePool release]",
            num_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # First stop is +[NSString stringWithFormat:].
        self.expect(
            "thread backtrace",
            "Stop at +[NSString stringWithFormat:]",
            substrs=["Foundation`+[NSString stringWithFormat:]"],
        )

        # Do the disassemble for the currently stopped function.
        self.runCmd("disassemble -f")

        self.runCmd("process continue")
        # Skip another breakpoint for +[NSString stringWithFormat:].
        self.runCmd("process continue")

        # Followed by a.out`-[MyString initWithNSString:].
        self.expect(
            "thread backtrace",
            "Stop at a.out`-[MyString initWithNSString:]",
            substrs=["a.out`-[MyString initWithNSString:]"],
        )

        # Do the disassemble for the currently stopped function.
        self.runCmd("disassemble -f")

        self.runCmd("process continue")

        # Followed by -[MyString description].
        self.expect(
            "thread backtrace",
            "Stop at -[MyString description]",
            substrs=["a.out`-[MyString description]"],
        )

        # Do the disassemble for the currently stopped function.
        self.runCmd("disassemble -f")

        self.runCmd("process continue")
        # Skip another breakpoint for -[MyString description].
        self.runCmd("process continue")

        # Followed by -[NSAutoreleasePool release].
        self.expect(
            "thread backtrace",
            "Stop at -[NSAutoreleasePool release]",
            substrs=["Foundation`-[NSAutoreleasePool release]"],
        )

        # Do the disassemble for the currently stopped function.
        self.runCmd("disassemble -f")



"""
Test Expression Parser code gen for ClassTemplateSpecializationDecl to insure
that we generate a TemplateTypeParmDecl in the TemplateParameterList for empty
variadic packs.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestClassTemplateSpecializationParametersHandling(TestBase):
    def test_class_template_specialization(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )
        self.expect_expr("b.foo()", result_type="int", result_value="1")



import inspect


def requires_self(func):
    func_argc = len(inspect.getfullargspec(func).args)
    if (
        func_argc == 0
        or (getattr(func, "im_self", None) is not None)
        or (hasattr(func, "__self__"))
    ):
        return False
    else:
        return True



"""
Test some expressions involving STL data types.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class STLTestCase(TestBase):
    @expectedFailureAll(bugnumber="llvm.org/PR36713")
    def test(self):
        """Test some expressions involving STL data types."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.cpp")
        )

        # Now try some expressions....

        self.runCmd(
            'expr for (int i = 0; i < hello_world.length(); ++i) { (void)printf("%c\\n", hello_world[i]); }'
        )

        self.expect("expr associative_array.size()", substrs=[" = 3"])
        self.expect("expr associative_array.count(hello_world)", substrs=[" = 1"])
        self.expect("expr associative_array[hello_world]", substrs=[" = 1"])
        self.expect('expr associative_array["hello"]', substrs=[" = 2"])

    @expectedFailureAll(
        compiler="icc",
        bugnumber="ICC (13.1, 14-beta) do not emit DW_TAG_template_type_parameter.",
    )
    @add_test_categories(["pyapi"])
    def test_SBType_template_aspects(self):
        """Test APIs for getting template arguments from an SBType."""
        self.build()
        (_, _, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.cpp")
        )
        frame0 = thread.GetFrameAtIndex(0)

        # Get the type for variable 'associative_array'.
        associative_array = frame0.FindVariable("associative_array")
        self.DebugSBValue(associative_array)
        self.assertTrue(associative_array, VALID_VARIABLE)
        map_type = associative_array.GetType()
        self.DebugSBType(map_type)
        self.assertTrue(map_type, VALID_TYPE)
        num_template_args = map_type.GetNumberOfTemplateArguments()
        self.assertGreater(num_template_args, 0)

        # We expect the template arguments to contain at least 'string' and
        # 'int'.
        expected_types = {"string": False, "int": False}
        for i in range(num_template_args):
            t = map_type.GetTemplateArgumentType(i)
            self.DebugSBType(t)
            self.assertTrue(t, VALID_TYPE)
            name = t.GetName()
            if "string" in name:
                expected_types["string"] = True
            elif "int" == name:
                expected_types["int"] = True

        # Check that both entries of the dictionary have 'True' as the value.
        self.assertTrue(all(expected_types.values()))



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCharTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self, dictionary=None):
        """These basic expression commands should work as expected."""
        self.build(dictionary=dictionary)

        lldbutil.run_to_source_breakpoint(
            self, "// Break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("foo(c)", result_value="1")
        self.expect_expr("foo(sc)", result_value="2")
        self.expect_expr("foo(uc)", result_value="3")
        self.expect_expr("g", result_type="char")
        self.expect_expr("gs", result_type="signed char")
        self.expect_expr("gu", result_type="unsigned char")

    def test_default_char(self):
        self.do_test()

    def test_signed_char(self):
        self.do_test(dictionary={"CFLAGS_EXTRAS": "-fsigned-char"})

    def test_unsigned_char(self):
        self.do_test(dictionary={"CFLAGS_EXTRAS": "-funsigned-char"})



"""Test SBThread.GetExtendedBacktraceThread API with queue debugging."""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestExtendedBacktraceAPI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.m"

    @skipUnlessDarwin
    @add_test_categories(["objc", "pyapi"])
    def test_extended_backtrace_thread_api(self):
        """Test GetExtendedBacktraceThread with queue debugging."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Get Xcode developer directory path.
        # Try DEVELOPER_DIR environment variable first, then fall back to xcode-select.
        xcode_dev_path = os.environ.get("DEVELOPER_DIR")

        if not xcode_dev_path:
            import subprocess

            xcode_dev_path = (
                subprocess.check_output(["xcode-select", "-p"]).decode("utf-8").strip()
            )

        # Check for libBacktraceRecording.dylib.
        libbtr_path = os.path.join(
            xcode_dev_path, "usr/lib/libBacktraceRecording.dylib"
        )

        self.assertTrue(
            os.path.isfile(libbtr_path),
            f"libBacktraceRecording.dylib is not present at {libbtr_path}",
        )

        self.assertTrue(
            os.path.isfile("/usr/lib/system/introspection/libdispatch.dylib"),
            "introspection libdispatch dylib not installed.",
        )

        # Create launch info with environment variables for libBacktraceRecording.
        launch_info = lldb.SBLaunchInfo(None)
        launch_info.SetWorkingDirectory(self.get_process_working_directory())
        launch_info.SetEnvironmentEntries(
            [
                f"DYLD_INSERT_LIBRARIES={libbtr_path}",
                "DYLD_LIBRARY_PATH=/usr/lib/system/introspection",
            ],
            True,
        )

        # Launch the process and run to breakpoint.
        target, process, thread, bp = lldbutil.run_to_name_breakpoint(
            self, "do_work_level_5", launch_info=launch_info, bkpt_module="a.out"
        )

        self.assertTrue(target.IsValid(), VALID_TARGET)
        self.assertTrue(process.IsValid(), PROCESS_IS_VALID)
        self.assertTrue(thread.IsValid(), "Stopped thread is valid")
        self.assertTrue(bp.IsValid(), VALID_BREAKPOINT)

        # Call GetNumQueues to ensure queue information is loaded.
        num_queues = process.GetNumQueues()

        # Check that we can find the com.apple.main-thread queue.
        main_thread_queue_found = False
        for i in range(num_queues):
            queue = process.GetQueueAtIndex(i)
            if queue.GetName() == "com.apple.main-thread":
                main_thread_queue_found = True
                break

        # Verify we have at least 5 frames.
        self.assertGreaterEqual(
            thread.GetNumFrames(),
            5,
            "Thread should have at least 5 frames in backtrace",
        )

        # Get frame 2 BEFORE calling GetExtendedBacktraceThread.
        # This mimics what Xcode does - it has the frame objects ready.
        frame2 = thread.GetFrameAtIndex(2)
        self.assertTrue(frame2.IsValid(), "Frame 2 is valid")

        # Now test GetExtendedBacktraceThread.
        # This is the critical part - getting the extended backtrace calls into
        # libBacktraceRecording which does an inferior function call, and this
        # invalidates/clears the unwinder state.
        extended_thread = thread.GetExtendedBacktraceThread("libdispatch")

        # This should be valid since we injected libBacktraceRecording.
        self.assertTrue(
            extended_thread.IsValid(),
            "Extended backtrace thread for 'libdispatch' should be valid with libBacktraceRecording loaded",
        )

        # The extended thread should have frames.
        self.assertGreater(
            extended_thread.GetNumFrames(),
            0,
            "Extended backtrace thread should have at least one frame",
        )

        # Test frame 2 on the extended backtrace thread.
        self.assertGreater(
            extended_thread.GetNumFrames(),
            2,
            "Extended backtrace thread should have at least 3 frames to access frame 2",
        )

        extended_frame2 = extended_thread.GetFrameAtIndex(2)
        self.assertTrue(extended_frame2.IsValid(), "Extended thread frame 2 is valid")

        # NOW try to access variables from frame 2 of the ORIGINAL thread.
        # This is the key test - after GetExtendedBacktraceThread() has executed
        # an inferior function call, the unwinder state may be invalidated.
        # Xcode exhibits this bug where variables show "register fp is not available"
        # after extended backtrace retrieval.

        # Set frame 2 as the selected frame so expect_var_path works.
        thread.SetSelectedFrame(2)

        variables = frame2.GetVariables(False, True, False, True)
        self.assertGreater(
            variables.GetSize(), 0, "Frame 2 should have at least one variable"
        )

        # Test all variables in frame 2, like Xcode does.
        # Use expect_var_path to verify each variable is accessible without errors.
        for i in range(variables.GetSize()):
            var = variables.GetValueAtIndex(i)
            var_name = var.GetName()

            # This will fail if the variable contains "not available" or has errors.
            self.expect_var_path(var_name)



"""Test settings and readings of program variables."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SetValuesTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.line1 = line_number("main.c", "// Set break point #1.")
        self.line2 = line_number("main.c", "// Set break point #2.")
        self.line3 = line_number("main.c", "// Set break point #3.")
        self.line4 = line_number("main.c", "// Set break point #4.")
        self.line5 = line_number("main.c", "// Set break point #5.")

    def test(self):
        """Test settings and readings of program variables."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Set breakpoints on several places to set program variables.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line1, num_expected_locations=1, loc_exact=True
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line2, num_expected_locations=1, loc_exact=True
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line3, num_expected_locations=1, loc_exact=True
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line4, num_expected_locations=1, loc_exact=True
        )

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line5, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # main.c:15
        # Check that 'frame variable --show-types' displays the correct data
        # type and value.
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(char) i = 'a'",
        )

        # Now set variable 'i' and check that it is correctly displayed.
        self.runCmd("expression i = 'b'")
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(char) i = 'b'",
        )

        self.runCmd("continue")

        # main.c:36
        # Check that 'frame variable --show-types' displays the correct data
        # type and value.
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[r"\((short unsigned int|unsigned short)\) i = 33"],
        )

        # Now set variable 'i' and check that it is correctly displayed.
        self.runCmd("expression i = 333")
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[r"\((short unsigned int|unsigned short)\) i = 333"],
        )

        self.runCmd("continue")

        # main.c:57
        # Check that 'frame variable --show-types' displays the correct data
        # type and value.
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(long) i = 33",
        )

        # Now set variable 'i' and check that it is correctly displayed.
        self.runCmd("expression i = 33333")
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(long) i = 33333",
        )

        self.runCmd("continue")

        # main.c:78
        # Check that 'frame variable --show-types' displays the correct data
        # type and value.
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(double) i = 2.25",
        )

        # Now set variable 'i' and check that it is correctly displayed.
        self.runCmd("expression i = 1.5")
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(double) i = 1.5",
        )

        self.runCmd("continue")

        # main.c:85
        # Check that 'frame variable --show-types' displays the correct data
        # type and value.
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(long double) i = 2.25",
        )

        # Now set variable 'i' and check that it is correctly displayed.
        self.runCmd("expression i = 1.5")
        self.expect(
            "frame variable --show-types",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(long double) i = 1.5",
        )



"""
Test how many times newly loaded binaries are notified;
they should be delivered in batches instead of one-by-one.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipUnlessPlatform(["linux"] + lldbplatformutil.getDarwinOSTriples())
class ModuleLoadedNotifysTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # At least DynamicLoaderDarwin and DynamicLoaderPOSIXDYLD should batch up
    # notifications about newly added/removed libraries.  Other DynamicLoaders may
    # not be written this way.
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// breakpoint")

    def setup_test(self, solibs):
        if lldb.remote_platform:
            path = lldb.remote_platform.GetWorkingDirectory()
            for f in solibs:
                lldbutil.install_to_target(self, self.getBuildArtifact(f))
        else:
            path = self.getBuildDir()
            if self.dylibPath in os.environ:
                sep = self.platformContext.shlib_path_separator
                path = os.environ[self.dylibPath] + sep + path
        self.runCmd(
            "settings append target.env-vars '{}={}'".format(self.dylibPath, path)
        )
        self.default_path = path

    def test_launch_notifications(self):
        """Test that lldb broadcasts newly loaded libraries in batches."""

        expected_solibs = [
            "lib_a." + self.platformContext.shlib_extension,
            "lib_b." + self.platformContext.shlib_extension,
            "lib_c." + self.platformContext.shlib_extension,
            "lib_d." + self.platformContext.shlib_extension,
        ]

        self.build()
        self.setup_test(expected_solibs)

        exe = self.getBuildArtifact("a.out")
        self.dbg.SetAsync(False)

        listener = self.dbg.GetListener()
        listener.StartListeningForEventClass(
            self.dbg,
            lldb.SBTarget.GetBroadcasterClassName(),
            lldb.SBTarget.eBroadcastBitModulesLoaded
            | lldb.SBTarget.eBroadcastBitModulesUnloaded,
        )

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # break on main
        breakpoint = target.BreakpointCreateByName("main", "a.out")

        event = lldb.SBEvent()
        # CreateTarget() generated modules-loaded events; consume them & toss
        while listener.GetNextEvent(event):
            True

        error = lldb.SBError()
        flags = target.GetLaunchInfo().GetLaunchFlags()
        process = target.Launch(
            listener,
            None,  # argv
            None,  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working directory
            flags,  # launch flags
            False,  # Stop at entry
            error,
        )  # error

        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        total_solibs_added = 0
        total_solibs_removed = 0
        total_modules_added_events = 0
        total_modules_removed_events = 0
        already_loaded_modules = []
        max_solibs_per_event = 0
        max_solib_chunk_per_event = []
        while listener.GetNextEvent(event):
            if lldb.SBTarget.EventIsTargetEvent(event):
                if event.GetType() == lldb.SBTarget.eBroadcastBitModulesLoaded:
                    solib_count = lldb.SBTarget.GetNumModulesFromEvent(event)
                    total_modules_added_events += 1
                    total_solibs_added += solib_count
                    added_files = []
                    for i in range(solib_count):
                        module = lldb.SBTarget.GetModuleAtIndexFromEvent(i, event)
                        # On macOS Ventura and later, dyld and the main binary
                        # will be loaded again when dyld moves itself into the
                        # shared cache. Use the basename so this also works
                        # when reading dyld from the expanded shared cache.
                        exe_basename = lldb.SBFileSpec(exe).basename
                        if module.file.basename not in ["dyld", exe_basename]:
                            self.assertNotIn(
                                module,
                                already_loaded_modules,
                                "{} is already loaded".format(module),
                            )
                        already_loaded_modules.append(module)
                        added_files.append(module.GetFileSpec().GetFilename())
                    if self.TraceOn():
                        # print all of the binaries that have been added
                        print("Loaded files: %s" % (", ".join(added_files)))

                    # We will check the latest biggest chunk of loaded solibs.
                    # We expect all of our solibs in the last chunk of loaded modules.
                    if solib_count >= max_solibs_per_event:
                        max_solib_chunk_per_event = added_files.copy()
                        max_solibs_per_event = solib_count

                if event.GetType() == lldb.SBTarget.eBroadcastBitModulesUnloaded:
                    solib_count = lldb.SBTarget.GetNumModulesFromEvent(event)
                    total_modules_removed_events += 1
                    total_solibs_removed += solib_count
                    if self.TraceOn():
                        # print all of the binaries that have been removed
                        removed_files = []
                        i = 0
                        while i < solib_count:
                            module = lldb.SBTarget.GetModuleAtIndexFromEvent(i, event)
                            removed_files.append(module.GetFileSpec().GetFilename())
                            i = i + 1
                        print("Unloaded files: %s" % (", ".join(removed_files)))

        # This is testing that we get back a small number of events with the loaded
        # binaries in batches.  Check that we got back more than 1 solib per event.
        # In practice on Darwin today, we get back two events for a do-nothing c
        # program: a.out and dyld, and then all the rest of the system libraries.
        # On Linux we get events for ld.so, [vdso], the binary and then all libraries,
        # but the different configurations could load a different number of .so modules
        # per event.
        self.assertLessEqual(set(expected_solibs), set(max_solib_chunk_per_event))



import lldb
import time
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestRegDefinitionInParts(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @skipIfRemote
    def test(self):
        """
        Test that lldb correctly fetches the target definition file
        in multiple chunks if the remote server only provides the
        content in small parts, and the small parts it provides is
        smaller than the maximum packet size that it declared at
        the start of the debug session.  qemu does this.
        """

        class MyResponder(MockGDBServerResponder):
            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                              <!DOCTYPE feature SYSTEM "gdb-target.dtd">
                              <target version="1.0">
                              <architecture>i386:x86-64</architecture>
                              <xi:include href="i386-64bit-core.xml"/>
                              </target>""",
                        False,
                    )

                if annex == "i386-64bit-core.xml" and offset == 0:
                    return (
                        """<?xml version="1.0"?>
<!-- Copyright (C) 2010-2015 Free Software Foundation, Inc.

     Copying and distribution of this file, with or without modification,
     are permitted in any medium without royalty provided the copyright
     notice and this notice are preserved.  -->

<!DOCTYPE feature SYSTEM "gdb-target.dtd">
<feature name="org.gnu.gdb.i386.core">
  <flags id="i386_eflags" size="4">
    <field name="CF" start="0" end="0"/>
    <field name="" start="1" end="1"/>
    <field name="PF" start="2" end="2"/>
    <field name="AF" start="4" end="4"/>
    <field name="ZF" start="6" end="6"/>
    <field name="SF" start="7" end="7"/>
    <field name="TF" start="8" end="8"/>
    <field name="IF" start="9" end="9"/>
    <field name="DF" start="10" end="10"/>
    <field name="OF" start="11" end="11"/>
    <field name="NT" start="14" end="14"/>
    <field name="RF" start="16" end="16"/>
    <field name="VM" start="17" end="17"/>
    <field name="AC" start="18" end="18"/>
    <field name="VIF" start="19" end="19"/>
    <field name="VIP" start="20" end="20"/>
    <field name="ID" start="21" end="21"/>
  </flags>

  <reg name="rax" bitsize="64" type="int64"/>
  <reg name="rbx" bitsize="64" type="int64"/>
  <reg name="rcx" bitsize="64" type="int64"/>
  <reg name="rdx" bitsize="64" type="int64"/>
  <reg name="rsi" bitsize="64" type="int64"/>
  <reg name="rdi" bitsize="64" type="int64"/>
  <reg name="rbp" bitsize="64" type="data_ptr"/>
  <reg name="rsp" bitsize="64" type="data_ptr"/>
  <reg name="r8" bitsize="64" type="int64"/>
  <reg name="r9" bitsize="64" type="int64"/>
  <reg name="r10" bitsize="64" type="int64"/>
  <reg name="r11" bitsize="64" type="int64"/>
  <reg name="r12" bitsize="64" type="int64"/>
  <reg name="r13" bitsize="64" type="int64"/>
  <reg name="r14" bitsize="64" type="int64"/>
  <reg name="r15" bitsize="64" type="int64"/>

  <reg name="rip" bitsize="64" type="code_ptr"/>
  <reg name="eflags" bitsize="32" type="i386_eflags"/>
  <reg name="cs" bitsize="32" type="int32"/>
  <reg name="ss" bitsize="32" ty""",
                        True,
                    )

                if annex == "i386-64bit-core.xml" and offset == 2045:
                    return (
                        """pe="int32"/>
  <reg name="ds" bitsize="32" type="int32"/>
  <reg name="es" bitsize="32" type="int32"/>
  <reg name="fs" bitsize="32" type="int32"/>
  <reg name="gs" bitsize="32" type="int32"/>

  <reg name="st0" bitsize="80" type="i387_ext"/>
  <reg name="st1" bitsize="80" type="i387_ext"/>
  <reg name="st2" bitsize="80" type="i387_ext"/>
  <reg name="st3" bitsize="80" type="i387_ext"/>
  <reg name="st4" bitsize="80" type="i387_ext"/>
  <reg name="st5" bitsize="80" type="i387_ext"/>
  <reg name="st6" bitsize="80" type="i387_ext"/>
  <reg name="st7" bitsize="80" type="i387_ext"/>

  <reg name="fctrl" bitsize="32" type="int" group="float"/>
  <reg name="fstat" bitsize="32" type="int" group="float"/>
  <reg name="ftag" bitsize="32" type="int" group="float"/>
  <reg name="fiseg" bitsize="32" type="int" group="float"/>
  <reg name="fioff" bitsize="32" type="int" group="float"/>
  <reg name="foseg" bitsize="32" type="int" group="float"/>
  <reg name="fooff" bitsize="32" type="int" group="float"/>
  <reg name="fop" bitsize="32" type="int" group="float"/>
</feature>""",
                        False,
                    )

                return None, False

            def readRegister(self, regnum):
                return ""

            def readRegisters(self):
                return "0600000000000000c0b7c00080fffffff021c60080ffffff1a00000000000000020000000000000078b7c00080ffffff203f8ca090ffffff103f8ca090ffffff3025990a80ffffff809698000000000070009f0a80ffffff020000000000000000eae10080ffffff00000000000000001822d74f1a00000078b7c00080ffffff0e12410080ffff004602000011111111222222223333333300000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000007f0300000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000801f0000"

            def haltReason(self):
                return "T02thread:dead;threads:dead;"

            def qfThreadInfo(self):
                return "mdead"

            def qC(self):
                return ""

            def qSupported(self, client_supported):
                return "PacketSize=1000;qXfer:features:read+"

            def QThreadSuffixSupported(self):
                return "OK"

            def QListThreadsInStopReply(self):
                return "OK"

        self.server.responder = MyResponder()
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            time.sleep(10)
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        target = self.dbg.CreateTargetWithFileAndArch(None, None)

        process = self.connect(target)

        if self.TraceOn():
            interp = self.dbg.GetCommandInterpreter()
            result = lldb.SBCommandReturnObject()
            interp.HandleCommand("target list", result)
            print(result.GetOutput())

        rip_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("rip")
        self.assertEqual(rip_valobj.GetValueAsUnsigned(), 0x00FFFF800041120E)

        ss_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("ss")
        self.assertEqual(ss_valobj.GetValueAsUnsigned(), 0x22222222)

        if self.TraceOn():
            print("rip is 0x%x" % rip_valobj.GetValueAsUnsigned())
            print("ss is 0x%x" % ss_valobj.GetValueAsUnsigned())



"""Look up type information for typedefs of same name at different lexical scope and check for correct display."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TypedefTestCase(TestBase):
    @expectedFailureAll(compiler="clang", bugnumber="llvm.org/pr19238")
    def test_typedef(self):
        """Test 'image lookup -t a' and check for correct display at different scopes."""
        self.build()
        self.image_lookup_for_multiple_typedefs()

    def image_lookup_for_multiple_typedefs(self):
        """Test 'image lookup -t a' at different scopes and check for correct display."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        typearray = (
            "float",
            "float",
            "char",
            "double *",
            "float",
            "int",
            "double",
            "float",
            "float",
        )
        arraylen = len(typearray) + 1
        for i in range(1, arraylen):
            loc_line = line_number("main.c", "// Set break point " + str(i) + ".")
            lldbutil.run_break_set_by_file_and_line(
                self, "main.c", loc_line, num_expected_locations=1, loc_exact=True
            )

        self.runCmd("run", RUN_SUCCEEDED)

        for t in typearray:
            self.expect(
                "thread list",
                STOPPED_DUE_TO_BREAKPOINT,
                substrs=["stopped", "stop reason = breakpoint"],
            )
            self.expect(
                "image lookup -t a",
                DATA_TYPES_DISPLAYED_CORRECTLY,
                substrs=['name = "' + t + '"'],
            )
            self.runCmd("continue")



import gdbremote_testcase
import random
import socket
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbgdbserverutils import Server
import lldbsuite.test.lldbplatformutil
from lldbgdbserverutils import Pipe


class TestGdbRemoteConnection(gdbremote_testcase.GdbRemoteTestCaseBase):
    SHARED_BUILD_TESTCASE = False

    @skipIfRemote  # reverse connect is not a supported use case for now
    def test_reverse_connect(self):
        # Reverse connect is the default connection method.
        self.connect_to_debug_monitor()
        # Verify we can do the handshake.  If that works, we'll call it good.
        self.do_handshake()

    @skipIfRemote
    def test_named_pipe(self):
        family, type, proto, _, addr = socket.getaddrinfo(
            self.stub_hostname, 0, proto=socket.IPPROTO_TCP
        )[0]
        self.sock = socket.socket(family, type, proto)
        self.sock.settimeout(self.DEFAULT_TIMEOUT)

        self.addTearDownHook(lambda: self.sock.close())

        pipe = Pipe(self.getBuildDir())

        self.addTearDownHook(lambda: pipe.close())

        args = self.debug_monitor_extra_args
        if lldb.remote_platform:
            args += ["*:0"]
        else:
            args += ["localhost:0"]

        args += ["--named-pipe", pipe.name]

        server = self.spawnSubprocess(
            self.debug_monitor_exe, args, install_remote=False
        )

        pipe.finish_connection(self.DEFAULT_TIMEOUT)
        port = pipe.read(10, self.DEFAULT_TIMEOUT)
        # Trim null byte, convert to int
        addr = (addr[0], int(port[:-1]))
        self.sock.connect(addr)
        self._server = Server(self.sock, server)

        # Verify we can do the handshake.  If that works, we'll call it good.
        self.do_handshake()



"""
Test thread states.
"""


import unittest
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ThreadStateTestCase(TestBase):
    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="llvm.org/pr15824 thread states not properly maintained",
    )
    @skipIfDarwin  # llvm.org/pr15824 thread states not properly maintained and <rdar://problem/28557237>
    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureNetBSD
    def test_state_after_breakpoint(self):
        """Test thread state after breakpoint."""
        self.build()
        self.thread_state_after_breakpoint_test()

    @skipIfDarwin  # 'llvm.org/pr23669', cause Python crash randomly
    @expectedFailureAll(
        oslist=lldbplatformutil.getDarwinOSTriples(), bugnumber="llvm.org/pr23669"
    )
    def test_state_after_continue(self):
        """Test thread state after continue."""
        self.build()
        self.thread_state_after_continue_test()

    @skipIfDarwin  # 'llvm.org/pr23669', cause Python crash randomly
    @expectedFailureDarwin("llvm.org/pr23669")
    @expectedFailureNetBSD
    # This actually passes on Windows on Arm but it's hard to describe that
    # and xfail it everywhere else.
    @skipIfWindows
    # thread states not properly maintained
    @unittest.expectedFailure  # llvm.org/pr16712
    def test_state_after_expression(self):
        """Test thread state after expression."""
        self.build()
        self.thread_state_after_expression_test()

    # thread states not properly maintained
    @unittest.expectedFailure  # llvm.org/pr15824 and <rdar://problem/28557237>
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24668: Breakpoints not resolved correctly",
    )
    @skipIfDarwin  # llvm.org/pr15824 thread states not properly maintained and <rdar://problem/28557237>
    @expectedFailureNetBSD
    def test_process_state(self):
        """Test thread states (comprehensive)."""
        self.build()
        self.thread_states_test()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers for our breakpoints.
        self.break_1 = line_number("main.cpp", "// Set first breakpoint here")
        self.break_2 = line_number("main.cpp", "// Set second breakpoint here")

    def thread_state_after_breakpoint_test(self):
        """Test thread state after breakpoint."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        bp = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Make sure the thread is in the stopped state.
        self.assertTrue(
            thread.IsStopped(), "Thread state isn't 'stopped' during breakpoint 1."
        )
        self.assertFalse(
            thread.IsSuspended(), "Thread state is 'suspended' during breakpoint 1."
        )

        # Kill the process
        self.runCmd("process kill")

    def wait_for_running_event(self, process):
        listener = self.dbg.GetListener()
        lldbutil.expect_state_changes(self, listener, process, [lldb.eStateRunning])

    def thread_state_after_continue_test(self):
        """Test thread state after continue."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_2, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Continue, the inferior will go into an infinite loop waiting for
        # 'g_test' to change.
        self.dbg.SetAsync(True)
        self.runCmd("continue")
        self.wait_for_running_event(process)

        # Check the thread state. It should be running.
        self.assertFalse(
            thread.IsStopped(), "Thread state is 'stopped' when it should be running."
        )
        self.assertFalse(
            thread.IsSuspended(),
            "Thread state is 'suspended' when it should be running.",
        )

        # Go back to synchronous interactions
        self.dbg.SetAsync(False)

        # Kill the process
        self.runCmd("process kill")

    def thread_state_after_expression_test(self):
        """Test thread state after expression."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_2, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Get the inferior out of its loop
        self.runCmd("expression g_test = 1")

        # Check the thread state
        self.assertTrue(
            thread.IsStopped(),
            "Thread state isn't 'stopped' after expression evaluation.",
        )
        self.assertFalse(
            thread.IsSuspended(),
            "Thread state is 'suspended' after expression evaluation.",
        )

        # Let the process run to completion
        self.runCmd("process continue")

    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24668: Breakpoints not resolved correctly",
    )
    @skipIfDarwin  # llvm.org/pr15824 thread states not properly maintained and <rdar://problem/28557237>
    @no_debug_info_test
    def test_process_interrupt(self):
        """Test process interrupt and continue."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        bpno = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Remove the breakpoint to avoid the single-step-over-bkpt dance in the
        # "continue" below
        self.assertTrue(target.BreakpointDelete(bpno))

        # Continue, the inferior will go into an infinite loop waiting for
        # 'g_test' to change.
        self.dbg.SetAsync(True)
        self.runCmd("continue")
        self.wait_for_running_event(process)

        # Go back to synchronous interactions
        self.dbg.SetAsync(False)

        # Stop the process
        self.runCmd("process interrupt")

        self.assertStopReason(thread.GetStopReason(), lldb.eStopReasonSignal)

        # Get the inferior out of its loop
        self.runCmd("expression g_test = 1")

        # Run to completion
        self.runCmd("continue")

    def thread_states_test(self):
        """Test thread states (comprehensive)."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_1, num_expected_locations=1
        )
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.break_2, num_expected_locations=1
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNotNone(thread)

        # Make sure the thread is in the stopped state.
        self.assertTrue(
            thread.IsStopped(), "Thread state isn't 'stopped' during breakpoint 1."
        )
        self.assertFalse(
            thread.IsSuspended(), "Thread state is 'suspended' during breakpoint 1."
        )

        # Continue, the inferior will go into an infinite loop waiting for
        # 'g_test' to change.
        self.dbg.SetAsync(True)
        self.runCmd("continue")
        self.wait_for_running_event(process)

        # Check the thread state. It should be running.
        self.assertFalse(
            thread.IsStopped(), "Thread state is 'stopped' when it should be running."
        )
        self.assertFalse(
            thread.IsSuspended(),
            "Thread state is 'suspended' when it should be running.",
        )

        # Go back to synchronous interactions
        self.dbg.SetAsync(False)

        # Stop the process
        self.runCmd("process interrupt")

        self.assertStopReason(thread.GetState(), lldb.eStopReasonSignal)

        # Check the thread state
        self.assertTrue(
            thread.IsStopped(), "Thread state isn't 'stopped' after process stop."
        )
        self.assertFalse(
            thread.IsSuspended(), "Thread state is 'suspended' after process stop."
        )

        # Get the inferior out of its loop
        self.runCmd("expression g_test = 1")

        # Check the thread state
        self.assertTrue(
            thread.IsStopped(),
            "Thread state isn't 'stopped' after expression evaluation.",
        )
        self.assertFalse(
            thread.IsSuspended(),
            "Thread state is 'suspended' after expression evaluation.",
        )

        self.assertStopReason(thread.GetState(), lldb.eStopReasonSignal)

        # Run to breakpoint 2
        self.runCmd("continue")

        self.assertStopReason(thread.GetState(), lldb.eStopReasonBreakpoint)

        # Make sure both threads are stopped
        self.assertTrue(
            thread.IsStopped(), "Thread state isn't 'stopped' during breakpoint 2."
        )
        self.assertFalse(
            thread.IsSuspended(), "Thread state is 'suspended' during breakpoint 2."
        )

        # Run to completion
        self.runCmd("continue")

        # At this point, the inferior process should have exited.
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbpexpect import PExpectTest


class EditlineCompletionsTest(PExpectTest):
    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_completion_truncated(self):
        """Test that the completion is correctly truncated."""
        self.launch(dimensions=(10, 20))
        self.child.send("_regexp-\t")
        self.child.expect("        _regexp-a...")
        self.child.expect("        _regexp-b...")

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_description_truncated(self):
        """Test that the description is correctly truncated."""
        self.launch(dimensions=(10, 70))
        self.child.send("_regexp-\t")
        self.child.expect(
            "        _regexp-attach    -- Attach to process by ID or name."
        )
        self.child.expect(
            "        _regexp-break     -- Set a breakpoint using one of several..."
        )

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_separator_omitted(self):
        """Test that the separated is correctly omitted."""
        self.launch(dimensions=(10, 32))
        self.child.send("_regexp-\t")
        self.child.expect("        _regexp-attach   \r\n")
        self.child.expect("        _regexp-break    \r\n")

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_separator(self):
        """Test that the separated is correctly printed."""
        self.launch(dimensions=(10, 33))
        self.child.send("_regexp-\t")
        self.child.expect("        _regexp-attach    -- A...")
        self.child.expect("        _regexp-break     -- S...")

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_multiline_description(self):
        """Test that multi-line descriptions are correctly padded and truncated."""
        self.launch(dimensions=(10, 72))
        self.child.send("k\t")
        self.child.expect(
            "        kdp-remote -- Connect to a process via remote KDP server."
        )
        self.child.expect(
            "                      If no UDP port is specified, port 41139 is assu..."
        )
        self.child.expect(
            "                      kdp-remote is an abbreviation for 'process conn..."
        )
        self.child.expect("        kill       -- Terminate the current target process.")

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_completion_pagination(self):
        """Test that we use the terminal height for pagination."""
        self.launch(dimensions=(10, 30))
        self.child.send("_regexp-\t")
        self.child.expect("Available completions:")
        self.child.expect("        _regexp-attach")
        self.child.expect("        _regexp-break")
        self.child.expect("        _regexp-break-add")
        self.child.expect("        _regexp-bt")
        self.child.expect("        _regexp-display")
        self.child.expect("        _regexp-down")
        self.child.expect("        _regexp-env")
        self.child.expect("More")

    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_completion_multiline_pagination(self):
        """Test that we use the terminal height for pagination and account for multi-line descriptions."""
        self.launch(dimensions=(6, 72))
        self.child.send("k\t")
        self.child.expect("Available completions:")
        self.child.expect(
            "        kdp-remote -- Connect to a process via remote KDP server."
        )
        self.child.expect(
            "                      If no UDP port is specified, port 41139 is assu..."
        )
        self.child.expect(
            "                      kdp-remote is an abbreviation for 'process conn..."
        )
        self.child.expect("More")



import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ArrayTypedefTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_array_typedef(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )
        self.expect("expr str", substrs=['"abcd"'])
